      program hcs17_1
      
c Bootstrap simulation of harvest rules on a given population,
c Starts with priming with fixed F to get a population at equilibrium,
c or with initial numbers from a nin-file(numbers and variance-covariances)
c Then, runs for maxyr years with catches decided by a harvest rule
c Version 15_1 was a major update since previous versions- see manual.
c hcs17_1 has some improvements that have emerged since then.

c Developing of version 17_1 started 22/6-2017

c Some bugs were discovered May 2017: 
c - Some subroutines did not get obs numbers if the timing was such
c    that no projection was needed. Fixed.
c - An error in the file number for the compare basis print option. Fixed
c - In year0 subroutine, line 1549, the start of the loop should be 0, not 1. Fixed
c - Percentage rule rewritten 11/6-17, Should be right now.
c    Checked that previous results without stabilizers remain unchanged.

c The version of hcs15_1 developed for the sardine HCR evaluation May 2017
c has been further amended in June 2017 to simulate blue whiting,
c and is launced as a new version: hcs17_1. 

c That includes:
c   Autocorrelation in recruitment
c   Autocorrelation in obs error year factor
c   Printout of autocorrelations and CV for SSB in year 0
c   Revision of some harvest rule subroutines. 



c D.W.Skagen: Dankert Skagen Fisheries Science Consultant
c dankert@dwsk.net; www.dwsk.net

      include 'hcscom17_1.i'

c Initialise random seed
      rintvar=15349668.0d0
c Define and open files for input and output
c For bio.inn there are two alternative formats, 
c indicated in the first line on the file, 
c comment out the one you do not want.
c The file names are hard coded - if you want other names you have
c to change them here and recompile
      innfilb='bio.inn'
      innfilo='opt.inn'
      innfilc='canum.inn'
      open (30,file='kladd')
      open (33,file='inputlist')
c read input      
      call readpopinn
      call readoptinn
c prepare output files      
      call prepfiles
c initialize rymax
      do iy=1,maxyr
      rymax(iy)=-1.0
      enddo
c calculate yield/recruit
      call yieldrecr
c start loop over options
      call optruns
c Clear up      
      call closefiles
      end
      
      subroutine readpopinn
c Read the bio.inn file
      include 'hcscom17_1.i'
      character*200 streng,tempfile(200)
      character*1 intype
      real tempread(5)
      integer ireadcode(maxyr)
      write(33,*) 'Valid lines in bio.inn'
      open (10,file=innfilb)
c transfer to tempfile valid lines, starting in position 1
      line=0
100   read(10,'(a200)',end=101) streng
      call findstring(streng,ii)
      if (ii.gt.0) then
      line=line+1
      tempfile(line) = streng(1:200)
      write(33,*) line,tempfile(line)(1:50)
      endif
      goto 100
101   close (10)


c Read basic specs
      read(tempfile(1),*) iy0
      read(tempfile(2),*) lowage,iaold
      read(tempfile(3),*) iaref1,iaref2
      read(tempfile(4),*) itsbage
      read(tempfile(5),*) propf,propm
      read(tempfile(6),*) ispdelay
      if (iaold.gt.maxage) then
      write(*,*) 'Oldest age larger than maxage'
      write(*,*) 'Adjust maxage in hcscom17_1.i and recompile the progra
     &m'
      endif
c Recruitments
c Line with regime specs
      iflgread=0
      read(tempfile(7),*) nregimes
      if (nregimes.gt.1) then
      read(tempfile(7),*) idummy,icodeshift
       if (icodeshift.eq.1) then
        listreg(1)=1
        read(tempfile(7),*) dummy1, dummy2,(listreg(i),i=2,nregimes)
c An extra entry in listreg is filled out, to make subsequent loops easier.
        listreg(nregimes+1)=maxyr
       else if (icodeshift.eq.2) then
        read(tempfile(7),*) dummy1, dummy2,regintv,lastreg
       else
      write(*,*) 'Something wrong with specification of recruitment regi
     &mes'
      stop
      endif
      else
c case only one regime
      listreg(1)=1
      endif

c Read specs for each regime     
c Regime number, option for parameter specs: 1: Read here; 2: read from file
      ireadflag=0
      i=0
      read(tempfile(8),*) (ireadcode(ireg),ireg=1,nregimes)      
      do ireg=1,nregimes
      if (ireadcode(ireg).eq.1) then
      read(tempfile(5*i+9),*) (irecrcode(ireg,1,j),j=1,3)
c SR function, distribution type, truncation code      
      read(tempfile(5*i+10),*) (recrparam(ireg,1,j),j=1,6)
c asrparam,bsrparam,sigmar,rtrunc1,rtrunc2,acfactr (=autocorrelation)
      read(tempfile(5*i+11),*) (recrparam(ireg,1,j),j=7,9)
c ramplitude,rperiod,rphase
      read(tempfile(5*i+12),*) r1,r2,r3,r4,r5
      irecrcode(ireg,1,4) = int(r1+0.001)
      recrparam(ireg,1,10)=r2
      recrparam(ireg,1,11)=r3
      recrparam(ireg,1,12)=r4
      irecrcode(ireg,1,5) = int(r5+0.001)
c spasmodic option, spasm. interval, spasm. sigma, spasm factor, previous spasm year    
      read(tempfile(5*i+13),*) narrec(ireg,1),
     &(arrec(ireg,1,j),j=1,narrec(ireg,1))      
c autoregressive coefficients
      

c Fill all iterations equal to this one
      do j=2,niter
      do k=1,12
      recrparam(ireg,j,k)=recrparam(ireg,1,k)
      enddo
      do k=1,5
    
      irecrcode(ireg,j,k)=irecrcode(ireg,1,k)
      enddo
      do k=1,narrec(ireg,1)
      arrec(ireg,j,k)= arrec(ireg,1,k)
      enddo
      narrec(ireg,j)=narrec(ireg,1)
      enddo
     
      else if (ireadcode(ireg).eq.2) then
      ireadflag=1
c      write(*,*) 'ireadflag',ireg,ireadflag
      else
      write(*,*) 'Source for SR parameters not properloy specified'
      write(*,*) 'Should be 1 or 2'
      write(*,*) tempfile(8)
      endif
       i=i+1
      

      enddo
      ilast=5*i+8
      
c ioptspasm,rinterval,rintvcv,spasmfct,lastspasm
c SInce there are several sets of recruitment parameters, they are read to 
c arrays recrparam (for reals) and irecrcode (for integers).
c They can be read fro a file ar directly from bio.inn

c The numbering code is:
c irecrcode:
c 1. jfunc: The code for stock recruit function (see below>)
c 2. jdist: The code for distribution of noise around the S-R function
c 3. jtrunc: The code for truncation type
c 4. ioptspasm: Code for type of spasmodic function
c 5. lastspasm: Year where counting of spasmodic recruitments start
c recrparam:
c 1. asrparam: a-parameter in SR function
c 2. bparam: b-parameter in SR function
c 3. sirmar: dispersion parameter for noise around SR function
c 4. rtrunc1: lower level for truncation
c 5. rtrunc2: upper level for truncation
c 6. acfactr: Autocorrelation in recruitment
c 7. ramplitude: amplitude in periodic function
c 8. rperiod: period in periodic function
c 9. rphase: phase in periodic function
c 10. rinterval: Interval between spasmodic years
c 11. rintvcv: Dispersion parameter for spasmodic intervals
c 12. spasmfact: multiplier in spasmodic years
c
c key for recruitment models
c jfunc: 1: Hockey
c        2: Bev-Holt
c        3: Ricker
c  >9. Parameters from file
c jdist: 1. Normal distribution 
c        2. Lognormal distribution
c jtrunc: 1: xsi > rtrunc2 or xsi < rtrunc1: 
c         2: xm > rtrunc2 or xm < rtrunc1
c recruitment model parameters ( a and b parameters
c sigmar and truncs refer to noise multiplier
c Parameters in a periodic recruitment - ramplitude = 0 is flat
c R = R0*(1+ramplitude*cos(2pi(year-rphase)/rperiod))
c Spasmodic recruitments - sets up a list with good years (intervals lognorm random with CV), 
c spasmfct is a multiplier applied to those years. spasmfact=1.0 is neutral
c lastspasm is the last great year, relative to year 0, can be negative
c
c Case some regimes have parameters specified on a file
       if (ireadflag.gt.0) then
       open (12,file='SR-params')
221     read(12,*,end=222) ireg,iter,(tempread(i),i=1,5),
     &(recrparam(ireg,iter,i),i=1,12)
c      write(*,*) ireg,iter,(recrparam(ireg,iter,i),i=1,5)
      do i=1,5
      irecrcode(ireg,iter,i)=int(tempread(i)+0.0001)
      enddo
       goto 221
222     close(12)
       endif            


c Read common data for weight variation
      read(tempfile(ilast+1),*) wbref
      read(tempfile(ilast+2),*) wrref
      read(tempfile(ilast+3),*) wsigma
      read(tempfile(ilast+4),*) wampl,wperiod,wphase
      read(tempfile(ilast+5),*) wycllow,wyclhigh,wyclalfa
      ilast=ilast+5
      
c Ready if increment algorithm is actovated, see subroutine calcwema      
c      read(tempfile(ilast+6),*) wincrlow,wincrhigh,wincralfa
c      ilast=ilast+6

c Read type code for biological properties
      read(tempfile(ilast+1),*) i
      if (i.eq.1) then
      intype='i'
      else if (i.eq.2) then
      intype='p'
      else
      write(*,*) 'Incomprehensible specification of biological input'
      write(*,*) 'file format: ',ilast,intype
      write(*,*) 'Must be 1 or 2- correct the file and start again'
      stop
      endif
      ilast=ilast+1
c Intype = p: Read parameters for age dependent data, and generate the actual data from there.
      if (intype.eq.'p') then
c generate age-dependent data from growth model specified on the file 
      read(tempfile(ilast+1),*) rmpar
      read(tempfile(ilast+2),*) rlinf
      read(tempfile(ilast+3),*) rl0  
c guidance for growth-k: Z/1.7; Z = 2*M - 3*M
      read(tempfile(ilast+4),*) gk
      read(tempfile(ilast+5),*) condfact
      read(tempfile(ilast+6),*) slopemat
      read(tempfile(ilast+7),*) sel50
      read(tempfile(ilast+8),*) slopesel
c Density dependence parameters (common to all ages and types
      read(tempfile(ilast+9),*) factlow,facthigh,wmalfa
      ilast=ilast+9
c Finished reading for intype=p, calculate actual data:
      rkfact=1.0-exp(-gk)
c maturation at 2/3 of Linf
      rmat50=0.667*rlinf
      do 110 ia=lowage,iaold
      if (ia.eq.lowage) then
      rlact=rl0
      else
      rlact=(rlinf-rllast)*rkfact+rllast
      endif
      rllast=rlact
      rm(ia)=rmpar
      sel(ia,1)=(1.0+exp(-slopesel*(rlinf-sel50)))/
     &(1.0+exp(-slopesel*(rlact-sel50)))
      wecaspec(ia,1)=rlact**3*condfact
      westspec(ia,1)= wecaspec(ia,1)
      voksenspec(ia,1)= (1.0+exp(-slopemat*(rlinf-rmat50)))/
     &(1.0+exp(-slopemat*(rlact-rmat50)))
      wecaspec(ia,2)=factlow*wecaspec(ia,1)
      wecaspec(ia,3)=facthigh*wecaspec(ia,1)
      wecaspec(ia,4)=wmalfa
      wecaspec(ia,5)=0.0
      westspec(ia,2)=factlow*wecaspec(ia,1)
      westspec(ia,3)=facthigh*wecaspec(ia,1)
      westspec(ia,4)=wmalfa
      westspec(ia,5)=0.0
      voksenspec(ia,2)=factlow*voksenspec(ia,1)
      voksenspec(ia,3)=min(facthigh*voksenspec(ia,1),1.0)
      voksenspec(ia,4)=wmalfa
      voksenspec(ia,5)=0.0

110   continue     
 
      else
c Intype = i: read age-dependent data directly from the file
c First,  
      line= ilast    

c weca at age
      do 14 ia=lowage,iaold
      line=line+1
      read(tempfile(line),*) i,(wecaspec(ia,j),j=1,5)
      if (i.ne.ia) then
      write(*,*) 'Check ageing of biol. data input at ',i
      stop
      endif
c wecaspec is stated as relative on the file, convert to absolute.
      wecaspec(ia,2)=wecaspec(ia,2)*wecaspec(ia,1)      
      wecaspec(ia,3)=wecaspec(ia,3)*wecaspec(ia,1)      
14    continue 
      
c west at age
      do 15 ia=lowage,iaold
      line=line+1
      read(tempfile(line),*) i,(westspec(ia,j),j=1,5)
      if (i.ne.ia) then
      write(*,*) 'Check ageing of biol. data input at ',i
      stop
      endif
      westspec(ia,2)=westspec(ia,2)*westspec(ia,1)      
      westspec(ia,3)=westspec(ia,3)*westspec(ia,1)      
15    continue 
     
c proportion mature at age
      do 16 ia=lowage,iaold
      line=line+1
      read(tempfile(line),*) i,(voksenspec(ia,j),j=1,5)
      if (i.ne.ia) then
      write(*,*) 'Check ageing of biol. data input at ',i
      stop
      endif
      voksenspec(ia,2)=voksenspec(ia,2)*voksenspec(ia,1)      
      voksenspec(ia,3)=min(voksenspec(ia,3)*voksenspec(ia,1),1.0)
16    continue 

c natural mortality
      do 12 ia=lowage,iaold
      line=line+1
      read(tempfile(line),*) i,rm(ia)
      if (i.ne.ia) then
      write(*,*) 'Check ageing of biol. data input at ',i
      stop
      endif
12    continue 

c selection at age
      do 13 ia=lowage,iaold
      line=line+1
      read(tempfile(line),*) i,sel(ia,1)
      if (i.ne.ia) then
      write(*,*) 'Check ageing of biol. data input at ',i
      stop
      endif
13    continue 

      iend=(iaold-lowage+1)*5+ilast
      if (line.ne.iend) then
      write(*,*) 'WARNING!'
      write(*,*) 'The number of accepted lines on the bio.inn file'
      write(*,*) 'deviates from the number expetced'
      write(*,*) 'Expected',iend
      write(*,*) 'Found   ',line   
      write(*,*) 'Last accepted line read:',tempfile(line)
      endif
c end reading input for i or p options
      endif
c Prime weca etc. with standard values
      do ia=lowage,iaold
      weca(ia)=wecaspec(ia,1)      
      west(ia)=westspec(ia,1)    
      rmat(ia)=voksenspec(ia,1)     
      enddo 
c fill in for ages below lowage
      if (lowage.gt.0) then
      do 23 i=0,lowage-1
      rm(i)=0.0
      sel(i,1)=0.0
      weca(i)=0.0
      west(i)=0.0
      rmat(i)=0.0
23    continue
      endif  
c Transfer initial values for weights and maturities to earlier years
c for later reference
      do iy=-maxage,0
      do i=0,iaold
      trueweca(iy,i)=weca(i)
      truewest(iy,i)=west(i)
      truemat(iy,i)=rmat(i)
      enddo
      enddo
c normalize selection so that selection over (iaref1 - iaref2)=1
      avg=0.0
      do 20 ia=iaref1,iaref2
      sel(ia,2)=sel(ia,1)
      avg=avg+sel(ia,1)
20    continue
      avg=avg/float(iaref2-iaref1+1)
      do 21 ia=0,iaold
      sel(ia,1)=sel(ia,1)/avg
      sel(ia,2)=sel(ia,1)
21    continue   
      end

      
      subroutine readoptinn    
c Read management options        
      include 'hcscom17_1.i'
      character*200 streng,tempfile(200)
      write(33,*)
      write(33,*) 'Valid lines in opt.inn'
      open (10,file=innfilo)
c transfer to tempfile valid lines, starting in position 1
      line=0
100   read(10,'(a200)',end=101) streng
      call findstring(streng,ii)
      if (ii.gt.0) then
      line=line+1
      tempfile(line) = streng(ii:200)
      write(33,*) line,tempfile(line)(1:50)

      endif
      goto 100
101   close (10)
c Initialize optrange
      do i=1,nopt
      optrange(i,1)=0.0
      optrange(i,2)=0.0
      optrange(i,3)=1.0
      enddo

c Read block with fixed general options (lines 1-17)
      read(tempfile(1),*) idata
c idata: priming (1) or numbers from file (2-4).      
      if (idata.eq.1) then
c initial data from priming      
      read(tempfile(2),*) fprim     
      else if (idata.eq.2.or.idata.eq.3.or.idata.eq.4) then
c initial data from nin.inn file
c format numbers; vsv matrix
      read(tempfile(2),*) tac0
      call readnin
      endif

c fmax: maximum F that can be achieved. Protection of shortage of fish.
      read(tempfile(3),*) fmax
c lastassn: Time gap between assessment year and last year with 
c Normally negative, i.e. start projection through intermediate
c year in year iy+lastassn
      read(tempfile(4),*) lastassn
      iobsstart=lastassn
c Interval between decision years
      read(tempfile(5),*) idur
c Reference Blim (mean,sigma: )
      read(tempfile(6),*) blim
      read(tempfile(7),*) idistblim
      read(tempfile(8),*) sigmablim
c Youngest age in TSB
      read(tempfile(9),*) ibplusage
c Read gradual implementation of rule
      read(tempfile(10),*) iabrgrad
c Read ban-borrow factor      
      read(tempfile(11),*) bbfactor
c Read specification for reruitment in projection      
      read(tempfile(12),*) ioptdr1
      read(tempfile(13),*) ioptdr2
      read(tempfile(14),*) iydrec1
      if (iydrec1.lt.iobsstart) iobsstart=iydrec1
      read(tempfile(15),*) iydrec2
      if (iydrec2.lt.iydrec1) then
      write(*,*) 'last year before first year in recruitment spec.'
      write(*,*) 'correct lines 14 and 15 in opt.inn'
      stop
      endif
      
      read(tempfile(16),*) sigmadr
      read(tempfile(17),*) iydrec3

c harvest rule basis
      iproj=lastassn
      read(tempfile(18),*) nbasis
      line=19
      do i=1,nbasis
      read(tempfile(line),*) ibastable(i,1)
      line=line+1
      read(tempfile(line),*) ibastable(i,2)
      if (ibastable(i,2).lt.iobsstart) iobsstart=ibastable(i,2)
      line=line+1
      read(tempfile(line),*) ibastable(i,3)
      if (ibastable(i,3).gt.iproj) iproj=ibastable(i,3)
      line=line+1
      read(tempfile(line),*) ibastable(i,4)
      line=line+1
 
      enddo
c Read rule and rule measure
      read(tempfile(line),*) irule
      line=line+1
      read(tempfile(line),*) ibasrule
      line=line+1
c Decide number of rule parameters to read
      if (irule.eq.1) then
      npar=7
      else if (irule.eq.2) then
      npar=7
      else if (irule.eq.3) then
      npar=6
      else if (irule.eq.4) then
      npar=8
      else if (irule.eq.5) then
      npar=8
      else if (irule.eq.6) then
      npar=9
      else if (irule.eq.7) then
      npar=2
      else if (irule.eq.8) then
      npar=5
      else
      write(*,*) 'Asked for non-existing rule:', irule
      stop
      endif
c read rule parameters into optrange
c Note: in optrange the first 10 are allocated to rule parameters, only those used are filled in
      do i=1,npar
      read (tempfile(line),*) (optrange(i,j),j=1,3)
      line=line+1
      enddo
      
      
c Read constraint rules
c The 3 rules must be on the file, but can be given values that are 
c ignored
c Filter rule
      read(tempfile(line),*) icrule
      line=line+1
      if (icrule.ne.1) then
      write(*,*) 'something wrong with opt.inn near Constraint rule 1'
      stop
      endif
      do j=1,4
      read(tempfile(line),*) icbastable(1,j)
      line=line+1
      enddo
      if (icbastable(1,2).lt.iobsstart) iobsstart=icbastable(1,2)
      if (icbastable(1,3).lt.iproj) iproj=icbastable(1,3)
      read(tempfile(line),*) (optrange(11,j),j=1,3)
      line=line+1
      read(tempfile(line),*) (optrange(12,j),j=1,3)
      line=line+1
c Percentage rule
      read(tempfile(line),*) icrule
      line=line+1
      if (icrule.ne.2) then
      write(*,*) 'something wrong with opt.inn near Constraint rule 2'
      stop
      endif
      do j=1,4
      read(tempfile(line),*) icbastable(2,j)
      if (icbastable(2,2).lt.iobsstart) iobsstart=icbastable(2,2)
      if (icbastable(2,3).lt.iproj) iproj=icbastable(2,3)
      line=line+1
      enddo
      read(tempfile(line),*) (optrange(13,j),j=1,3)
      line=line+1
      read(tempfile(line),*) (optrange(14,j),j=1,3)
      line=line+1
c Min and max
      read(tempfile(line),*) icrule
      line=line+1
      if (icrule.ne.3) then
      write(*,*) 'something wrong with opt.inn near Constraint rule 3'
      stop
      endif
      read(tempfile(line),*) (optrange(15,j),j=1,3)
      line=line+1
      read(tempfile(line),*) (optrange(16,j),j=1,3)
      line=line+1
c Obs model noise

      read(tempfile(line),*) (optrange(17,j),j=1,3)
      line=line+1
      read(tempfile(line),*) (optrange(18,j),j=1,3)
      line=line+1  
      read(tempfile(line),*) (optrange(19,j),j=1,3)
      line=line+1

c Initialize the cvobs, to ensure that they have zero for non-recruited ages 
      do ia=0,maxage
      do j=1,2
      cvobs(ia,j)=0.0
      enddo
      cvimp(ia)=0.0
      enddo
      
      do ia=lowage,iaold
      read (tempfile(line),*) i,cvobs(ia,1)
      if (i.ne.ia) then
      write(*,*) 'Wrong ages for obs cv-values in the opt file'
      write(*,*) 'Stated as ',i,' Expected ',ia, 'line',line
      write(*,*) 'Correct the file and start again'
      stop
      endif
      line=line+1
      enddo
      
c read coefficients for autoregressive obs. error
c Initialize first. Dimension is maxage, more is never needed
      do j=0,maxage
      acfact(j)=0.0
      enddo
      read(tempfile(line),*) nac
      read(tempfile(line),*) ndummy,(acfact(j),j=0,nac-1)
c Input number of coefficients=0 is interpreted as one term = 1.
      if (nac.eq.0) then
      nac=1
      acfact(0)=1.0
      endif
      
      line=line+1

c Noise on observed catches
      read(tempfile(line),*) (optrange(20,j),j=1,3)
      line=line+1

      do ia=lowage,iaold
      read (tempfile(line),*) i,cvobs(ia,2)
      if (i.ne.ia) then
      write(*,*) 'Wrong ages for  catch cv-values in the opt file'
      write(*,*) 'Stated as ',i,' Expected ',ia,'line',line
      write(*,*) 'Correct the file and start again'
      stop
      endif
      line=line+1
      enddo
c Observed recruitment surveys      
      read(tempfile(line),*) (optrange(21,j),j=1,3)
      line=line+1
      read(tempfile(line),*) (optrange(22,j),j=1,3)
      line=line+1
c Observed biomass surveys      
      read(tempfile(line),*) (optrange(23,j),j=1,3)
      line=line+1
      read(tempfile(line),*) (optrange(24,j),j=1,3)
      line=line+1


c Implementation noise

      read(tempfile(line),*) (optrange(25,j),j=1,3)
      line=line+1
      read(tempfile(line),*) (optrange(26,j),j=1,3)
      line=line+1

      do ia=lowage,iaold
      read (tempfile(line),*) i,cvimp(ia)
      if (i.ne.ia) then
      write(*,*) 'Wrong ages for impl. cv-values in the opt file'
      write(*,*) 'Stated as ',i,' Expected ',ia,'line',line
      write(*,*) 'Correct the file and start again'
      stop
      endif
      line=line+1
      enddo

c adjust flag for projection
c iproj is determined already by the need for basis
c add one year if propf or propm >0
      if (propm.gt.0.0.or.propf.gt.0.0) iproj=iproj+1

c Read list of printout options
c This list, with reference to file numbers, declares if
c this kind of output shall be made. The files are always opened,
c to avoid accidental writing to files not open, but will
c not be written to if the rules are followed.    

c Note that file 14 is free at the moment!  
      do i=11,25
      read(tempfile(line),*) ii,j
      ioutopt(ii)=j
      if (ii.eq.12.and.j.ge.1) then
      ioutopt(12)=j
      do ij=1,j
      line=line+1
      read(tempfile(line),*) intvpr1(ij),intvpr2(ij)
      if (intvpr2(ij).gt.maxyr) then
      write(*,*) 'Warning: You have asked for a printout table for years
     &',intvpr1(ij),intvpr2(ij)
      write(*,*) 'That is outside the years covered in the simulations'
      intvpr2(ij)=maxyr
      if (intvpr1(ij).gt.maxyr) intvpr1(ij)=maxyr
      write(*,*) 'You get', intvpr1(ij),'-',intvpr2(ij)
      endif
      enddo
      endif
      line=line+1
      enddo

c If needed, read canum file
      if (iobsstart.lt.0) then
      do i=0,maxage
      do j=-maxage,maxyr+mbasis
      cobs(i,j)=0.0
      enddo
      enddo
      open (10,file=innfilc)
532   read(10,'(a200)',end=520) streng
      read(streng,*) i
      iy=i-iy0
      if (iy.ge.-maxage) 
     &read(streng,*) idummy,(cobs(ia,iy),ia=lowage,iaold)
      goto 532
520   close(10)
      endif
      
      end
     
     
      subroutine readnin
      include 'hcscom17_1.i'
      
      if (idata.eq.2) then
      open (31,file='nin.inn')
c read nin-file with vcv
c Note: The nin file is assumed to have plain numbers in the first column.
c In the matrix, the diagonal terms are covariances on the log scale,
c and the other entries are correlation coefficients.         
      do 500 ia=lowage,iaold
      read(31,*) i,rnin(ia),(vcv(ia,j),j=lowage,iaold)
      if (i.ne.ia) then
      write(*,*) 'Wrong ages on nin.inn-file'
      write(*,*) 'Stated as ',i,' Expected ',ia
      write(*,*) 'Correct the file'
      stop
      endif
500   continue    
      if (lowage.gt.0) then
      do 200 i=1,lowage
      read(31,*) pastssb(i-1)   
200   continue
      endif
      close (10)  

c check the vcv for symmetry
      do 501 i=lowage,iaold
      do 502 j=i,iaold
      if (vcv(i,j).ne.vcv(j,i)) then
      write(*,*) 'Correlation matrix in ninfile is not symmetric'
      write(*,*) i,j,vcv(i,j) 
      write(*,*) 'is different from '
      write(*,*) j,i,vcv(j,i)
      write(*,*) 'Correct the file'
      stop
      endif
502   continue
501   continue  
c convert correlations to covariances     
      do 110 i=lowage,iaold
      do 110 j=i+1,iaold
      vcv(i,j)=vcv(i,j)*vcv(i,i)*vcv(j,j)
      vcv(j,i)=vcv(i,j)
110   continue

                    
      
      else if (idata.eq.3) then
c if input form file with numbers only, linked to using the 
c obs-model for generating the starting numbers
      open (31,file='nin.obs')
c format: age, number at age
      do 251 ia=lowage,iaold
      read (31,*) i,r1
      if (i.ne.ia) then
      write(*,*) 'Wrong ages in the nin.obs file'
      write(*,*) 'Stated as ',i,' Expected ',ia
      write(*,*) 'Correct the file and start again'
      stop
      endif
      rnin(ia)=r1
251   continue
      if (lowage.gt.0) then
      do 202 i=1,lowage
      read(31,*) pastssb(i-1)   
202   continue
      endif
      close (31)  
      
      else if (idata.eq.4) then
            
      open (31,file='nin.lst')
252    read (31,*,end=253) i,(rninit(i,j),j=lowage,iaold)      
      goto 252
253    close(31)
      
      else
      
      write(*,*) 'Invalid option for year 0: ',idata
      stop
      endif
      end
       
      subroutine prepfiles
c opens files and writes headers for files that collect results from multiple runs
      include 'hcscom17_1.i'
      character*10 streng

      if (ioutopt(11).eq.1) open (11,file='Results')
      if (ioutopt(13).eq.1) open (13,file='Autoc_CVS0')
      if (ioutopt(14).eq.1) open (14,file='Dummy')
      if (ioutopt(15).eq.1) open (15,file='Traj')
      if (ioutopt(16).eq.1) open (16,file='Recovery')
      if (ioutopt(17).eq.1) open (17,file='Yieldrecr')
      if (ioutopt(18).eq.1) open (18,file='Catch_agedist')
      if (ioutopt(19).eq.1) open (19,file='SSB_agedist')
      if (ioutopt(20).eq.1) open (20,file='SRpairs')
      if (ioutopt(21).eq.1) open (21,file='S0_distribution')
      if (ioutopt(22).eq.1) open (22,file='F_distribution')
      if (ioutopt(23).eq.1) open (23,file='IAV_distribution')
      if (ioutopt(24).eq.1) open (24,file='Compare_basis')
      if (ioutopt(25).eq.1) open (25,file='Wema')
      if (ioutopt(26).eq.1) open (26,file='Comprecr')
      
      do jj=11,26
      enddo
      
c Prepare headers for printout, according to the harvest rule irule
       call header

c write headers for files collecting results from all runs      
     
      if (ioutopt(11).eq.1) open (41,file='MainResults')
      
      
      if (ioutopt(12).ge.1) then
      do iopt=1,ioutopt(12)
      inty1=intvpr1(iopt)
      inty2=intvpr2(iopt)      
      streng='Years___  '
      write(streng(6:7),'(i2)') inty1
      write(streng(9:10),'(i2)') inty2
      open (120+iopt,file=streng)
      write(120+iopt,*) headline(2)     
      enddo
      endif
      if (ioutopt(13).eq.1) write(13,*) headline(3)

      end

 
 
      
      subroutine closefiles
      include 'hcscom17_1.i'
      if (ioutopt(11).eq.1) close (11)
      if (ioutopt(11).eq.1) close (41)
      if (ioutopt(12).eq.1) close (12)
      if (ioutopt(13).eq.1) close (13)
      if (ioutopt(14).eq.1) close (14)
      if (ioutopt(15).eq.1) close (15)
      if (ioutopt(16).eq.1) close (16)
      if (ioutopt(17).eq.1) close (17)
      if (ioutopt(18).eq.1) close (18)
      if (ioutopt(19).eq.1) close (19)
      if (ioutopt(20).eq.1) close (20)
      if (ioutopt(21).eq.1) close (21)
      if (ioutopt(25).eq.1) close (25)
      if (ioutopt(26).eq.1) close (26)
c files 22,23 and 24 are closed by the yieldrecr routine
      close (30)
      close (33)
      end
      
 
      subroutine optruns
c Controls the screeing over options, calls bootruns
c for each new set of options
      include 'hcscom17_1.i'
      integer nsteps(nopt)
c check that no optrange is the wrong way around
      do i=1,nopt
      if (optrange(i,2).lt.optrange(i,1)) then
      write(*,*) 'Mis-specified options range, option',i
      write(*,*) (optrange(i,j),j=1,3)
      stop
      endif
      enddo
c convert optranges to integers
      do i=1,nopt
      if (optrange(i,3).gt.0.0) then
      nsteps(i)=int((optrange(i,2)*1.001-optrange(i,1))/optrange(i,3))
      else
      nsteps(i)=0
      endif
      enddo

      do 905 i1=0,nsteps(1)
      h1=optrange(1,1)+float(i1)*optrange(1,3)
      do 905 i2=0,nsteps(2)
      h2=optrange(2,1)+float(i2)*optrange(2,3)
      do 905 i3=0,nsteps(3)
      h3=optrange(3,1)+float(i3)*optrange(3,3)
      do 905 i4=0,nsteps(4)
      h4=optrange(4,1)+float(i4)*optrange(4,3)
      do 905 i5=0,nsteps(5)
      h5=optrange(5,1)+float(i5)*optrange(5,3)
      do 905 i6=0,nsteps(6)
      h6=optrange(6,1)+float(i6)*optrange(6,3)
      do 905 i7=0,nsteps(7)
      h7=optrange(7,1)+float(i7)*optrange(7,3)
      do 905 i8=0,nsteps(8)
      h8=optrange(8,1)+float(i8)*optrange(8,3)
      do 905 i9=0,nsteps(9)
      h9=optrange(9,1)+float(i9)*optrange(9,3)
      do 905 i10=0,nsteps(10)
      h10=optrange(10,1)+float(i10)*optrange(10,3)
      do 905 i11=0,nsteps(11)
      h11=optrange(11,1)+float(i11)*optrange(11,3)
      do 905 i12=0,nsteps(12)
      h12=optrange(12,1)+float(i12)*optrange(12,3)
      do 905 i13=0,nsteps(13)
      h13=optrange(13,1)+float(i13)*optrange(13,3)
      do 905 i14=0,nsteps(14)
      h14=optrange(14,1)+float(i14)*optrange(14,3)
      do 905 i15=0,nsteps(15)
      h15=optrange(15,1)+float(i15)*optrange(15,3)
      do 905 i16=0,nsteps(16)
      h16=optrange(16,1)+float(i16)*optrange(16,3)
      do 905 i17=0,nsteps(17)
      h17=optrange(17,1)+float(i17)*optrange(17,3)
      do 905 i18=0,nsteps(18)
      h18=optrange(18,1)+float(i18)*optrange(18,3)
      do 905 i19=0,nsteps(19)
      h19=optrange(19,1)+float(i19)*optrange(19,3)
      do 905 i20=0,nsteps(20)
      h20=optrange(20,1)+float(i20)*optrange(20,3)
      do 905 i21=0,nsteps(21)
      h21=optrange(21,1)+float(i21)*optrange(21,3)
      do 905 i22=0,nsteps(22)
      h22=optrange(22,1)+float(i22)*optrange(22,3)
      do 905 i23=0,nsteps(23)
      h23=optrange(23,1)+float(i23)*optrange(23,3)
      do 905 i24=0,nsteps(24)
      h24=optrange(24,1)+float(i24)*optrange(24,3)
      do 905 i25=0,nsteps(25)
      h25=optrange(25,1)+float(i25)*optrange(25,3)
      do 905 i26=0,nsteps(26)
      h26=optrange(26,1)+float(i26)*optrange(26,3)

      hcrparam(1)=h1
      hcrparam(2)=h2
      hcrparam(3)=h3
      hcrparam(4)=h4
      hcrparam(5)=h5
      hcrparam(6)=h6
      hcrparam(7)=h7
      hcrparam(8)=h8
      hcrparam(9)=h9
      hcrparam(10)=h10
      hcrparam(11)=h11
      hcrparam(12)=h12
      hcrparam(13)=h13
      hcrparam(14)=h14
      hcrparam(15)=h15
      hcrparam(16)=h16
      hcrparam(17)=h17
      hcrparam(18)=h18
      hcrparam(19)=h19
      hcrparam(20)=h20
      hcrparam(21)=h21
      hcrparam(22)=h22
      hcrparam(23)=h23    
      hcrparam(24)=h24    
      hcrparam(25)=h25    
      hcrparam(26)=h26    
       
      
c  For each option, initialise arrays for assembling results 
      do 331 iy=0,maxyr
      do 333 iter=1,niter
      truessb(iy,iter)=-1.0
      truecatch(iy,iter)=-1.0
      tac(iy,iter)=-1.0
      truef(iy,iter)=-1.0      
      obsssb(iy,iter)=-1.0
      obsf(iy,iter)=-1.0
      iflag(iy,iter)=0
333   continue      
331   continue

c write headers for output of individual runs
      if (ioutopt(11).eq.1) then 
      write(11,*) 'Options for this run:'
      write(11,*) headline(1)
      write(11,fmtline1) (hcrparam(i),i=1,nruleparam),
     &(hcrparam(i),i=11,26)
      write(41,*) 'Options for this run:'
      write(41,*) headline(1)
      write(41,fmtline1) (hcrparam(i),i=1,nruleparam),
     &(hcrparam(i),i=11,26)
      endif
      do 790 ifil=15,16
      if (ioutopt(ifil).eq.1) then
      write(ifil,*) 'Options for this run:'
      write(ifil,*) headline(1)
      write(ifil,fmtline1) (hcrparam(i),i=1,nruleparam),
     &(hcrparam(i),i=11,26)
      endif
790   continue          
      do 791 ifil=20,26
      if (ioutopt(ifil).eq.1) then
      write(ifil,*) 'Options for this run:'
      write(ifil,*) headline(1)
      write(ifil,fmtline1) (hcrparam(i),i=1,nruleparam),
     &(hcrparam(i),i=11,26)
      endif
791   continue    

      write(*,*) (hcrparam(ii),ii=1,nruleparam),(hcrparam(ii),ii=11,26)   

      call bootruns
      call bootprint
905   continue
      end

      
      
      subroutine header
c Make header line building blocks for all printouts 
c The headers are in two common strings: headline(1-2)
c also write format strings to be used in printouts:fmtline1, fmtline2
      include 'hcscom17_1.i'
      character*200 headopt,headcons,headuncert,headres,headres4
      character*8 fmttemp1
      character*36 fmttemp2
      character*27 fmttemp3
      character*20 fmttemp4
      
c List of headers for options
c   Rule options
      if (irule.eq.1) then
      headopt='    Btrig1      Btrig2       Target       Alfa1       Alf
     &a2    vmin   vmax'
      fmttemp1='(7f12.3,'
      nruleparam=7
      
      else if (irule.eq.2) then
      headopt='B0      Btrig1      Btrig2      Target      Level1 Alfa2 
     &     vmax'
      fmttemp1='(7f12.3'
      nruleparam=7
      
       else if (irule.eq.3) then
      headopt='Btrig1 Btrig2 Target Slope1   Slope2    vmax '
      fmttemp1='(6f12.3,'
      nruleparam=6
 
 
      else if (irule.eq.4) then
      headopt='Btrig1 Btrig2 v0 Alfa1 Alfa2 Gain  vmin  vmax'
      fmttemp1='(8f12.3'
      nruleparam=8
       
      else if (irule.eq.5) then
      headopt='Btrig1    Btrig2     Ref_R    Target_low    Target_high 
     &Alfa1   Alfa2  vmax '
      fmttemp1='(8f12.3,'
      nruleparam=8

      else if (irule.eq.6) then
      headopt='B0    Btrig1    Btrig2     Ref_R    Target_low    Target_ 
     &high   Alfa1   Alfa2  vmax '
      fmttemp1='(9f12.3,'
      nruleparam=9
      
      else if (irule.eq.7) then
      headopt='B1   B2'
      fmttemp1='(2f12.3,'
      nruleparam=2

      else if (irule.eq.8) then
      headopt='B1   B2    vlow   vstd   vhigh'
      fmttemp1='(5f12.3,'
      nruleparam=5
        
        
      endif


c Constraints
      headcons='Filt_trig Filter Perc_trig Percent MinTAC   MaxTAC'
c Uncertainties
      headuncert='Bias_obsmod CV_ycft_obs Autoc_obs CV_obscatch Qrecr_su
     &rv CV_recr_surv Q_biom_surv CV_biom_surv Bias_impl CV_impl'           
      fmttemp2='f10.1,f6.3,f10.1,f6.1,2f10.1,10f6.3'
c Results
      headres=' Cmean  C10  C50  C90 Fmean  F10  F50  F90  Smean  SSB10 
     & SSB50  SSB90  Tmean  TSB10  TSB50  TSB90 IAV  Risklim  Probtrig  
     &Crash'   
      fmttemp3=',4f10.1,4f6.3,8f10.1,4f8.3)'   
      headres4=' ST10  ST50  ST90  SOD10  SOD50  SOD90  R10 R50  R90   
     & S0mean S010 S050 S090 S0CV  S0confint'  
      fmttemp4=',9f7.2,4f10.1,2f7.3)'   
c write strings      
      write(headline(1),*)  trim(headopt),' ',trim(headcons),' ',
     &trim(headuncert)
      write(headline(2),*)  trim(headopt),' ',trim(headcons),' ',
     &trim(headuncert),' ',trim(headres)
      write(headline(3),*)  trim(headopt),' ',trim(headcons),' ',
     &trim(headuncert),' ',trim(headres4)
      write(fmtline1,*) trim(fmttemp1),trim(fmttemp2),')'
      write(fmtline2,*) trim(fmttemp1),trim(fmttemp2),trim(fmttemp3)
c      write(*,*) trim(fmttemp1),trim(fmttemp2),trim(fmttemp4)
      write(fmtline3,*) trim(fmttemp1),trim(fmttemp2),trim(fmttemp4)
      if (ioutopt(22).eq.1) write(22,*) 'Iter  Year Decided   Real'
      if (ioutopt(23).eq.1) write(23,*) 'Iter   Year     IAV' 
      if (ioutopt(24).eq.1) write(24,*) 'Year Each basis: True   Obs' 
      if (ioutopt(20).eq.1) write(20,*) 'Iter  Year     SSB      Recr'
      if (ioutopt(26).eq.1) write(26,*) 'Iter  Year     TrueR      ObsR'

     
      end
      
      
      subroutine bootruns      
c For the set of parameters decided in optruns, run niter iterations.      
      include 'hcscom17_1.i'

c Local variables.
      real rn(0:maxage),rnex(0:maxage),tactemp(0:maxyrp)
      real canum(0:maxage)

c start sequence of random numbers in the same place for each run      
      rintvar=15349668.0d0
      
c loop over niter iterations 
      do 999 iter=1,niter
c      do 999 iter=1,10
c      write(*,*) 'iter',iter

c prepare random sequences etc.
      call preprandom
c get initial numbers etc.  
      call initializerun
c Run through year 0 (TAC is input)  
      call year0

c Year loop: years 1-- maxyr
      do 20 iy=1,maxyr     
c Get get true ns at start of year iy in rn 
      do ia=0,iaold
      rn(ia) = truen(ia,iy)
      enddo 
c      write(*,*) 'tru-yloop',iy,(rn(ia),ia=0,3)
c fill in observed n's as seen in year iy    
      call obsmodel(rn,iy,1)
c      write(*,*) 'obs-yloop',iy,(obsn(ia,iy,iy),ia=0,3)

c generate survey data for year iy.  
      call getrecrsurvey(iy,rn(0))
      call getobssurvey(iy,rn,truef(iy,iter),2)

c call hcr if there is no tac yet for iy   
c Note: Internally, hcr uses the weights from last assessment year.
c The weights are reset to current year at the end of hcr.
      if (tac(iy,iter).lt.0.0) call hcr(tactemp,iy)
      tac(iy,iter)=tactemp(iy)
c Implement the TAC      

c bank-borrow factor
      bb=bbfactor
c Need obsssb, has to be generated here. Used to check that bb is legal,
c Redundant if bbfactor=0, but costs nothing, and can be useful for printout
      do ia=0,iaold
      rnex(ia)=projn(ia,iy)
      enddo
c Get the assumed F  year y, for the SSB
      call findf(rnex,tac(iy,iter),ftemp,jflag,1)
c Get the SSB, as criterium for banking and borrowing      
      call calcssb(rnex,s,ftemp,1)
      obsssb(iy,iter)=s
c Note: The criterium is blim, not any calculated basis
      if (obsssb(iy,iter).lt.blim) bb=0.0
c Set the tac with bb
      tacbb=(1.0-bb)*tac(iy,iter)+bb*tac(iy-1,iter)
c      write(*,*) 'tacbb',tacbb,tac(iy,iter),tac(iy-1,iter),rnex
c For the record, get the F corresponding to tacbb into obsf. 
c Obsf is the assumed f that corresponds to the finally decided catch.

      call findf(rnex,tacbb,obsf(iy,iter),jflag,1)
      
c Implement the tac, with bb       
      call implement(iy,tacbb,yci,rn,jflg,canum)   
c NOTE: The true catch if the TAC is too large for the stock, is the 
c realized catch at Fmax 
      truecatch(iy,iter)=yci
c Get observed catches by putting errors to the implemented catches
c These are the catches at age that are rememberd
      call getobscanum(canum,iy)

c get current true SSB 
      call calcssb(rn,s,truef(iy,iter),2)
      truessb(iy,iter)=s
      call calctsb(rn,tsb)
      truetsb(iy,iter)=tsb
c Get the 7+ percentage      
      summ1=0.0
      summ2=0.0
      do 102 ia=0,iaold
      if (ia.ge.itsbage) summ1=summ1+rn(ia)
102   if (ia.ge.7) summ2=summ2+rn(ia)
      rn7(iy,iter)=summ2/summ1*100.0
c iflag, i.e. crash is either if the TAC cannot be taken (jflg=1) and/or 
c if SSB is below 1/10 of Blim      
      if (truessb(iy,iter).lt.blim/100.0 
     &.or.jflg.eq.1) iflag(iy,iter)=1
     
c project into next year      
      if (iy.lt.maxyr) then
      call updatewema(iy,rn)
      call nextyear(rn,truef(iy,iter),s,2,ispdelay,iy) 
c      write(*,*) 'tru-next',iy,(rn(ia),ia=0,3)


c Note the yearclass factor for weights for the new year class
      wyclfct(iy+1)=wyclfct0
      do 22 ia=0,iaold
      weca(ia)=trueweca(iy+1,ia)
      west(ia)=truewest(iy+1,ia)
      rmat(ia)=truemat(iy+1,ia)
22    truen(ia,iy+1)=rn(ia)
      truerecr(iy+1,iter)=rn(lowage)


       endif
c end year loop
20    continue

c print things that are discarded after each iteration
      if (ioutopt(24).eq.1.and.iter.le.100) call printbas
      if (ioutopt(20).eq.1.and.iter.lt.100) call printsr
      if (ioutopt(25).eq.1.and.iter.lt.10) call printwema
      if (ioutopt(26).eq.1.and.iter.lt.20) call printrecr

c end iteration loop
999   continue
      end


      subroutine preprandom
      include 'hcscom17_1.i'
      real geodist(0:maxage)
      
c Build initial xsiobs and cobs
      biasobs=hcrparam(17)
      yfctobs=hcrparam(18)
      acfactobs=hcrparam(19)
c build a new vector of random values.
      do iy=-iaold,0
      x=rnx(rintvar)
      xsi=snrn(x)*yfctobs
c include autocorrelation (acfactobs and xsiycftobs(iy)     are commons
      if (iy.gt.-iaold)
     &xsi=acfactobs*xsiycftobs(iy-1)+xsi*sqrt(1.0-acfactobs**2)
      xsiycftobs(iy)=xsi
      btemp=(1.0+biasobs)*exp(xsi-yfctobs**2/2.0)
c btemp is the common random multiplier
c By age
      do ia=0,iaold
      if (ia.lt.lowage) then
      xsiobs(ia,iy)=1.0
      else
      if (cvobs(ia,1).gt.0.0) then
      x=rnx(rintvar)
      xsi=snrn(x)
      xsi=xsi*cvobs(ia,1)
      else
      xsi=0.0
      endif
      xsiobs(ia,iy)=btemp*exp(xsi-cvobs(ia,1)**2/2.0)
      endif
      enddo
      enddo

     
c setup recruitment regimes
      if (nregimes.le.1) then
c case no regimeshifts
      do i=-maxage,maxyr
      nregact(i,iter)=1
      enddo
      else 
c nregimes >1
c Set up sequence of regimes for recruitment
c If icodeshift=1, listreg is ready from input
c If icodeshift=2, listreg is set up here      
       if (icodeshift.eq.2) then   
        call creategeo(geodist,regintv,maxyr)
        listreg(1)=1
        ishift=lastreg
        nn=1
c Find next shift
288     x=rnx(rintvar)
        ii=1
287     ii=ii+1
        if (geodist(ii).ge.x) goto 289
        if (ii.lt.maxyr) goto 287
289     ishift=ishift+ii
        if (ishift.gt.0.and.ishift.le.maxyr.and.nn.le.maxyr) then
         nn=nn+1
         listreg(nn)=ishift
        endif
        if (ishift.lt.maxyr) goto 288
        listreg(nn+1)=maxyr

        else 
c icodeshift not 2       
         if (icodeshift.ne.1) then
        
         write(*,*) 'Incomprehensible code for regime shift', icodeshift
         stop
        endif 
       endif
      endif
c Known shift years are now in listreg (fixed or derived from flex model)
c
c Set up nregact: Regime by year.
c Always start with regime 1, so prime nregact with that
c Also, years prior to year 0 (sometimes needed) are regime 1.
      do i=-maxage,maxyr
      nregact(i,iter)=1 
      enddo  
      if (nregimes.gt.1) then
       j=1
c for each regime block, define the years
c The last entry on listreg shall be maxyr
501   j=j+1
      iyfirst=listreg(j)
      iylast=listreg(j+1)-1
      if (listreg(j+1).eq.maxyr) iylast=maxyr
c Then, find the regime number k for those years
      if (icodeshift.eq.1) then
c Fixed sequence: Consecutive regimes
      k=j
      else if (icodeshift.eq.2) then
c Random regimes
c Find which regime k in this interval, each is equally likely
      x=rnx(rintvar)
      x=x*float(nregimes)
      k=int(x)+1      
c put in place
      endif
      do iy=iyfirst,iylast
      nregact(iy,iter)=k
      enddo
c end loop over regimes
      if (iylast.lt.maxyr) goto 501
      endif



c set up the sequence of spasmodic recruitment successes.  
c Get parameters for the regime in question
      do i=-maxage,maxyr
      ispasm(i)=0
      enddo
c set up spasmodic years for each regime
      do k=1,nregimes
      rinterval=recrparam(k,iter,10)
      rintvcv=recrparam(k,iter,11)
      spasmfct=recrparam(k,iter,12)
      ioptspasm=irecrcode(k,iter,4)
      if (k.eq.1) then
       iystart=-lowage
      else
      iystart=listreg(k)
      endif
       if (k.eq.nregimes) then
      iyend=maxyr
      else
      iyend=listreg(k+1)
      endif
      lastspasm=iystart+irecrcode(k,iter,5)
      if (rinterval.gt.1.0.and.spasmfct.gt.1.0) then
c only if the interval and spasmodic factor is greater than 1, 
c if not, no such effects are assumed

      if (ioptspasm.eq.1) then
c case spasms with random intervals
      rilast=float(lastspasm)
199   x=rnx(rintvar)
      xsi=snrn(x)
      ri=rinterval*exp(rintvcv*xsi)+rilast
      j=int(ri+0.5)
      if (j.le.iyend.and.j.ge.iystart) then
c Note: The spasmodic years start in lastspasm
c Years prior to year 0 only matter when priming the stock.
c Else, earlier years are only used to propagate the sequence. 
      ispasm(j)=1
      rilast=ri
      goto 199
      endif
      else if (ioptspasm.eq.2) then

       pr=1.0/rinterval
       ilast=lastspasm
297    x=rnx(rintvar)
       ii=0
       px=0.0
299    ii=ii+1
       flii=float(ii)-1.0
       px=px+pr*exp(flii*log(1.0-pr))
       if (px.ge.x) then
        goto 298
       else 
        if (ii.lt.maxyr) then
         goto 299
        else
         goto 298
        endif
       endif
298   j=ii+ilast
      ilast=j   
       if (j.le.iyend.and.j.ge.iystart) then
c Note: The spasmodic years start in lastspasm
c Years prior to year 0 only matter when priming the stock.
c Else, earlier years are only used to propagate the sequence. 
        ispasm(j)=1
        goto 297
       endif

       else if (ioptspasm.eq.3) then
c Flat distribution between rinterval*(1-cv,1+cv)
       ilast=lastspasm
398    x=rnx(rintvar)
       x=rinterval*(x*rintvcv+1.0-rintvcv/2.0)
       ii=int(x+0.5)+ilast
        if (ii.lt.maxyr) then
        if (ii.ge.-maxage) ispasm(ii)=1
        ilast=ii
        
         goto 398
        endif
       endif
c end if spasmodic recrutimets
      endif
      
      
c end regimes loop
      enddo

 
c build recruitment multipliers for the sequence of years
c transfer current recruitment parameters to working variables
      do iy=-lowage,maxyr
      ireg=nregact(iy,iter)
c case taken from normal input      
      asrorg=recrparam(ireg,iter,1)   
      bsrparam=recrparam(ireg,iter,2)   
      sigmar=recrparam(ireg,iter,3)   
      jfunc=irecrcode(ireg,iter,1)
      jdist=irecrcode(ireg,iter,2)
      rtrunc1=recrparam(ireg,iter,4)   
      rtrunc2=recrparam(ireg,iter,5)   
      acfactr=recrparam(ireg,iter,6)   
      ramplitude=recrparam(ireg,iter,7)   
      rperiod=recrparam(ireg,iter,8)   
      rpahse=recrparam(ireg,iter,9)   
      rinterval=recrparam(ireg,iter,10)   
      rintvcv=recrparam(ireg,iter,11)   
      spasmfct=recrparam(ireg,iter,12)   
      jtrunc=irecrcode(ireg,iter,3)
      
c periodic variation: period_factor(y)
      if (rperiod.gt.0.0) then
      period_factor(iy)=1.0+ramplitude*
     &cos(6.28*(float(iy)-rphase)/rperiod)
      else
      period_factor(iy)=1.0
      endif
c spasmodic kicks: spasm_factor(y)
      if (ispasm(iy).eq.1) then
      spasm_factor(iy) = spasmfct
      else
      spasm_factor(iy)=1.0
      endif
      
c stochasatic noise
c random       
10    x=rnx(rintvar)
      xsi1=snrn(x)*sigmar
c include autocorrelation 
      if (iy.gt.-lowage) then
      xsi2=acfactr*xsir2(iy-1)+xsi1*sqrt(1.0-acfactr**2)
      xsir2(iy)=xsi2
      else
      xsir2(iy)=xsi1
      endif
      
c autoregressive model
      xsir3(iy)=xsir2(iy)
      if (narrec(ireg,iter).gt.0) then
      do j=1,narrec(ireg,iter)
      xsir3(iy)=xsir3(iy)+arrec(ireg,iter,j)*xsir3(iy-j)      
      enddo
      endif
c convert to temporary multiplier xm
      if (jdist.eq.1) then
c Normal error, ps is CV 
      xm=1.0+xsir3(iy)
      else if (jdist.eq.2) then
      xm=exp(xsir3(iy)-sigmar**2/2.0)
      else
      write(*,*) 'Incomprehensible recruit distribution', jdist
      endif

c Truncation: Redraw (goto 10) if outside bounds.
c Variant 1: Trunc is lower and upper number of SDs (lower negative).
c of the primary standard normal number (
      if (jtrunc.eq.1) then
       if (xsi1.lt.rtrunc1.or.xsi1.gt.rtrunc2) then
        goto 10
        endif
      else
c Variant 2:  upper and lower bounds on the multiplier,
       if (xm.lt.rtrunc1.or.xm.gt.rtrunc2) then
        goto 10
       endif
      endif
c Final nummber
      rmult(iy,1)=period_factor(iy)
      rmult(iy,2)=spasm_factor(iy)
      rmult(iy,3)=xm
      enddo
      end
           

      subroutine initializerun
c Initializes variables, gets initial numbers from file
c or by priming as needed, runs through year 0.
c Time sequences for random variables that are independent of the state of the 
c stock are set up by a call to initrandom.
c At return, everytning is ready for simulating from year 1 onwards.      
      include 'hcscom17_1.i'
c Local variables.
      real rn(0:maxage),rn1(0:maxage),rn2(0:maxage),cc(maxage,maxage)
c Initialise arrays      
      do ia=0,iaold
      rn(ia)=-1.0
      rn1(ia)=-1.0
      weca(ia)=trueweca(0,ia)
      west(ia)=truewest(0,ia)
      rmat(ia)=truemat(0,ia)
      do iy=-maxage,maxyr
      truen(ia,iy)=-1.0
      projn(ia,iy)=-1.0
      wyclfct(iy)=1.0
      biosurv(iy,1)=-1.0
      biosurv(iy,2)=-1.0
      enddo
      enddo
      do iy=0,maxyr
      tac(iy,iter) = -1.0
      enddo
c get initial data      
      if (idata.eq.1) then
      call prime(rn)
      
      else if (idata.eq.2) then
c nin with vcv      
      do i=lowage,iaold
      rn1(i-lowage+1)=log(rnin(i))
      do j=lowage,iaold
      if (i.eq.j) then
      cc(i-lowage+1,i-lowage+1)=cvnin(i)
      else
      cc(i-lowage+1,j-lowage+1)=0.0
      endif
      enddo
      enddo
      n=iaold-lowage+1      
      call multnorm(rn1,cc,rn2,rintvar,n)
      do i=lowage,iaold
      j=i-lowage+1
c translate to plain numbers and include bias correction      
      rn(i)=exp(rn2(j)-vcv(i,i)**2/2.0)
      enddo

 
      else if (idata.eq.3) then
c nin with obsmodel
c Change the direction of observation bias. If we think the stock is 
c underestimated, we should start with higher values.
c Later, the assumed stock numbers should be smaller than the true ones.
      hcrparam(17)=-hcrparam(17)
      call obsmodel(rnin,0,0)
      hcrparam(17)=-hcrparam(17)
      do ia=0,maxage
      rn(ia)=obsn(ia,0,0)
      projn(ia,0)=rn(ia)
      enddo
      
      
      else if (idata.eq.4) then
      do ia=0,maxage
      rn(ia)=rninit(iter,ia)
      projn(ia,0)=rn(ia)
      enddo      
      else
      write(*,*) 'Incomprehensible code for initial numbers 0 ',idata
      stop
      endif
      
      
      if (idata.gt.1.and.lowage.gt.0) then
c get recruits for year classes younger than recruitment age if 
c recruitmnet age is above 0
c not needed when priming
      do ia=0,lowage-1
      iyr=-ia
      s=pastssb(iyr)
      call recruitment(rcr,iyr,s,1)
      rn(ia)=rcr
      projn(ia,0)=rn(ia)
c Use the yearclass factor for weights for the new year class
      wyclfct(iyr)=wyclfct0
      enddo
      endif

c Keep rn as true for the start of year 0
      do 11 ia=0,iaold      
11    truen(ia,0)=rn(ia)
      truerecr(0,iter)=rn(0)
      end


      
      subroutine year0
c Takes the stock through year 0      
      include 'hcscom17_1.i'
      real rn(0:maxage),rn1(0:maxage),canum(0:maxage)
      
      iy=0
c Transfer true n's to working array rn
c rn does not change in this routine, until nextyear is called
      do ia=0,iaold
      rn(ia)=truen(ia,iy)
      enddo      
c get TAC in year 0 = y0 and f = f0
c If priming, find the TAC from the priming F
      if (idata.eq.1) then
      f0=fprim
      call yield(rn,y0,f0,1)
      else if (idata.gt.1) then
c If nin, TAC constraint is assumed in year 0 
c Then, find the corresponding true F
      y0=tac0     
      call findf(rn,y0,f0,jflg,1)
      else
      write(*,*) 'Stop: Exploitation in year 0: missing specification'
      stop
      endif   
      
c Implement the TAC, get true catch, F SSB and project true Ns      
      call implement(iy,y0,yci,rn,jflg,canum)   
      truecatch(0,iter)=yci
  
c put true values in place  
c truen is already in place.
      call calcssb(rn,s,f0,1)
      truessb(0,iter)=s
      call calctsb(rn,tsb)
      truetsb(0,iter)=tsb
      tac(0,iter)=y0

c Get observed catches by putting errors to the implemented catches
      call getobscanum(canum,0)

c get distorted data for year 0 
      
      call obsmodel(rn,0,1)
      call getrecrsurvey(iy,rn(0))
      call getobssurvey(iy,rn,f0,1)
c find F and SSB according to assumed exploitation and distorted Ns      
      if (idata.ge.2) then
      do ia=0,iaold
      rn1(ia)=obsn(ia,0,0)
      enddo
      call findf(rn1,y0,fshit,jflg1,1)
      obsf(0,iter)=fshit
      call calcssb(rn1,sshit,fshit,1)
      else
      obsf(0,iter)=f0
      call calcssb(rn1,sshit,f0,1)
      endif
      obsssb(0,iter)=sshit  
c project true stock into next year  
      call updatewema(iy,rn)
      call nextyear(rn,f0,s,2,ispdelay,iy)
      do 12 ia=0,iaold
12    truen(ia,1)=rn(ia)
c      write(*,*) 'True year 0-> 1',(truen(ia,1),ia=0,3)
      truerecr(1,iter)=rn(0)
c Use the yearclass factor for weights for the new year class
      wyclfct(iy+1)=wyclfct0
c      write(30,*) 'N0',iter,(rn1(ii),ii=0,6)

      end


      subroutine hcr(tactemp,iy)
c generate TACs for year iy (and further if multiannual TACs)
c in tactemp, based on obssurviv and weights and maturities in the last assessment year
      include 'hcscom17_1.i'
c Local variables.
      real 
     &tactemp(0:maxyrp),tacout(0:maxyrp),
     &tacprev(0:maxyrp),tempc(0:maxyrp),
     &basis(mbasis)
c temporary TACs are defined maxyrp beyond last year, to allow projections that far.     
      integer iy
c fill in the working array sy and tactemp with old values as far as they go      
      do 40 i=0,iy-1
      tactemp(i)=tac(i,iter)
      tacout(i)=0.0
40    continue  
c Prepare initial values etc.
      do 10 i=iy,iy+iproj
      tactemp(i)=tactemp(i-1)
10    continue
      do 133 i=iy+lastassn,iy+iproj
      tacprev(i)=tactemp(i)
      tacout(i)=0.0
133    continue   

c set weights and maturities to the last assessment year
      do ia=lowage,iaold
      weca(ia)=trueweca(iy+lastassn,ia)
      west(ia)=truewest(iy+lastassn,ia)
      rmat(ia)=truemat(iy+lastassn,ia)
      enddo      
      
c prepare for prjojection if needed
c      if (iproj.gt.lastassn) then
c      call prepproj(iy)     
c get the starting numbers for both cases  
c Redundant, each routine that needs
c these numbers find them itself
c      do ia=0,iaold
c      rn(ia)=projn(ia,iy+lastassn)
c      enddo
c      else
c      do ia=0,iaold
c      rn(ia)=obsn(ia,iy+lastassn,iy+lastassn)
c      enddo

c      endif
      
      call decideprimtac(iy,basis,tactemp,v)
      if (iproj.gt.0) then
c check that the outcome of the rule is in accordance with the tacs
c criterium: v does not change
      nr=0
      v1=v
      
100   nr=nr+1
      v2=v1
      v1=v
      ssq=0.0    
      call decideprimtac(iy,basis,tactemp,v)
      if (v1.gt.0.0.and.v.gt.0.0) then
      ssq=(log(v1/v))**2
      else if (v1.le.0.0.and.v.eq.0.0) then
c exception handling if one of the v's is 0
c Both =0: Keep the v
      ssq=0.0
      else
c One =0, the other not 0: set ssq=1 (>0)
      ssq=1.0
      endif
      
      if (ssq.gt.0.000001) then
      if (nr.lt.5) then 
      goto 100
      else
c Her kommer søkerutine hvis fixpunkt-iterasjonen ikke går,
      vin=(v+v1)/2.0
      call getvfromv(iy,basis,tactemp,vin,vout)
      v1=vin
      v=vout
c break at nr=30, prsumably loop because of discontinuity
c caused by rounding of f that goes into project      
      if (nr.gt.30) then
      write(*,*) 'Eternal loop in search for v that reproduces itself in
     & subroutine hcr'
      else if (abs((v-vin)/vin).gt.0.000001) then
      goto 100
      endif
c endif nr.lt.5  
      endif  
c endif søk (ssq.gt.0.00001)
      endif
c end if iproj,gt,0
      endif
      
      t1=tactemp(iy)
      c1= basis(1)
      
c We now have the TAC according to the basic rule in tactemp.
c Step 2: Apply stabilizers
c 1. Apply filter if asked for
      if (hcrparam(12).gt.0.0.and.iy.gt.0) then
c Put the filtered tacs in tempc and check
      tempc(iy)=tactemp(iy)*(1.0-hcrparam(12))+
     &tac(iy-1,iter)*hcrparam(12)
c Fill in if further years
      if (idur.gt.1) then
      if (iabrgrad.eq.1) then
      do i=iy,iy+idur
      tempc(i)=tempc(iy)
      enddo
      else if (iabrgrad.eq.2) then
      diff=(tactemp(iy)-tactemp(iy-1))/float(idur)
      tempc(iy-1)=tactemp(iy-1)
      do i=iy,iy+idur
      tempc(i)=tempc(i-1)+diff
      enddo
      endif
    
      endif
c then check the criterium
      call buildcbasis(cbasis,1,iy)
c Shall we use the filter, now represented in tempc?
      if (icbastable(1,4).eq.3) then
c if the cbasis is a trend, use filter if the trend is smaller than the criterium
      if (cbasis.lt.hcrparam(11)) then
      do i=iy,iy+idur
      tactemp(i)=tempc(i)
      enddo
      endif
      else
c in other cases, use the filter if the cbasis is above the criterium
      if (cbasis.gt.hcrparam(11)) then

      do i=iy,iy+idur
      tactemp(i)=tempc(i)
      enddo
      endif
  
c end  if (icbastable(1,4).eq.3
      endif
c end filter  
      endif
      t2=tactemp(iy)

c 2. Does percentage constraint apply?
      percent=hcrparam(14)/100.0
c Just to make sure that tempc has a fresh value.      
      tempc(iy)=tactemp(iy)
      if (percent.gt.0.0.and.iy.gt.0) then  
c if so, the rule can be relevant
c Can the rule be used?
       if (tac(iy-1,iter).gt.0.0) then
c Last TAC was positive, so the percentage constraint can be used

c calculate the constrained TAC in tempc:
c First, find the ratio     
       actratio=tactemp(iy)/tac(iy-1,iter)-1.0  
             
        if (abs(actratio).gt.percent) then
c Change is big enough, find modified TAC
c cratio is the multiplier for change
         if (actratio.lt.0.0) then
           cratio=(1.0-percent)
         else
           cratio=(1.0+percent)      
         endif
        tempc(iy)=tac(iy-1,iter)*cratio
             
c Then, shall we use the constrained TAC or is there a Derogation?
c If there is a no derogation, put tempc into tactemp
      if (hcrparam(13).gt.0.0) then
c use constraint if criterium fulfilled. If so, tactemp is put (back) in tempc   
      call buildcbasis(cbasis,2,iy)
      c3= cbasis
      
      if (icbastable(2,4).eq.4) then
c if the cbasis is a trend,, accept constraint if the trend is below
c the criterium
      if (cbasis.lt.hcrparam(13)) tactemp(iy)=tempc(iy)
      else
c in other cases, accept the constraint if the cbasis is above the criterium
      if (cbasis.gt.hcrparam(13)) tactemp(iy)=tempc(iy)
      endif
      else
c case derogation bound =0, always constraint
      tactemp(iy)=tempc(iy)     
      
      endif
c endif over er for if (hcrparam(13).gt.0.0)      
      endif
c endif over er for if (tac(iy-1,iter).gt.0.0)   
      endif
c endif over er for if       (abs(actratio).gt.percent) 
      endif
c endif over er for if (percent.gt.0.0.and.iy.gt.0) 

      do i=iy,iy+idur
      tactemp(i)=tactemp(iy)
      enddo
      diff=0.0
      if (tac(iy-1,iter).gt.0.0.and.iy.gt.0)
     &diff=tactemp(iy)/tac(iy-1,iter)-1.0  
c      write(30,*) 'checkperc',hcrparam(12),hcrparam(13),hcrparam(14),
c     &hcrparam(15),iter,iy,t1,tactemp(iy),diff,c1
       
c Check min and max TAC
      do i=iy,iy+idur-1
      if (hcrparam(15).gt.tactemp(i)) tactemp(i)=hcrparam(15)
      if (hcrparam(16).lt.tactemp(i)) tactemp(i)=hcrparam(16)
      enddo
c reset weights and maturities
      do ia=lowage,iaold
      weca(ia)=trueweca(iy,ia)
      west(ia)=truewest(iy,ia)
      rmat(ia)=truemat(iy,ia)
      enddo
c keep basis1 for printout
      do i=1,nbasis
      decbas(iy,i,iter)=basis(i)
      enddo
       
      end       
 
 
      subroutine decideprimtac(iy,basis,tactemp,v)
c get the primary TAC according to the basic rule, for year iy,
c and optionally further years
      
      include 'hcscom17_1.i'
c Local variables.
c rn is from the outset the stock numbers at the start of the projection, 
c for example in iy+lastassn
      real rny(0:maxage),tactemp(0:maxyrp),
     &tacout(0:maxyrp),basis(mbasis)
      real v
      integer iy,nr,iflg
      nr=0
c Find the reference n-numbers for year iy to convert v to TAC  
c The numbers rny come from projn if projection, else from obsn    
      if (iproj.ge.0) then
      call project(tactemp,tacout,iy,iflg)
      do ia=0,iaold
      rny(ia)=projn(ia,iy)
c      write(*,*) iy,ia,'outprojn',projn(ia,iy)
      enddo
      else
      do ia=0,iaold
      rny(ia)=obsn(ia,iy,iy)
      enddo
      endif
c      write(*,*) 'SJEKK RNY',iy,iproj,(rny(ii),ii=0,3)
      
c For the record, keep the assumed recruitment in year iy      
      obsrecr(iy,iter)=rny(lowage)
      

c Build basis  
      call buildbasis(iy,basis)
c      write(*,*) 'basis',iy

c call the rule
      call rule(v,basis)
      
c translate this v into TAC      
      if (ibasrule.eq.1) then
c case v is an F-value
c      write(*,*) 'til yield',(rny(ii),ii=0,3)
      call yield(rny,y,v,1)
      else if (ibasrule.eq.2) then
c case v is a HR rule based  on an assessment
       call calcbplus(rny,tsb)
       y=v*tsb
      else if (ibasrule.eq.3) then
c case v is a TAC rule
      y=v
      else if (ibasrule.eq.4) then
      
c Surveys are for the year before the TAC year. Other choices
c require hard coding here
c case v is a HR, based on an SSB survey
       y=v*biosurv(iy-1,1)
      else if (ibasrule.eq.5) then
c case v is a HR, based on an TSB survey
       y=v*biosurv(iy-1,2)
      else if (ibasrule.eq.6) then
      y=v*tac(iy-1,iter)
      endif
c      write(*,*) 'rule',iy,basis(1),basis(2),v,y
c set the decided TAC for the whole decision period
      tactemp(iy)=y
      step=(tactemp(iy)-tactemp(iy-1))/float(idur)
      do  i=iy,iy+idur-1
      if (idur.gt.1.and.iabrgrad.eq.2) then
      tactemp(i)=tactemp(i-1)+step
      else
      tactemp(i)=tactemp(iy)
      endif
      enddo  
      end                   
            
            
            
            
      subroutine getvfromv(iy,basis,tactemp,vin,vout)
c get the primary TAC according to the input vin,
c and get the v = vout corresponding to that TAC.
c Adjust tactemp to vout
      
      include 'hcscom17_1.i'
c Local variables.
      real rny(0:maxage),tactemp(0:maxyrp),
     &tacout(0:maxyrp),basis(mbasis)
      real vin,vout
      integer iy,iflg
      
c translate the input vin into TAC      
      if (ibasrule.eq.1) then
c case v is an F-value
      call yield(rny,y,vin,1)
      else if (ibasrule.eq.2) then
c case v is a HR rule based  on an assessment
       call calcbplus(rny,tsb)
       y=vin*tsb
      else if (ibasrule.eq.3) then
c case v is a TAC rule
      y=vin
      else if (ibasrule.eq.4) then
c case v is a HR, based on an SSB survey
       y=vin*biosurv(iy-1,1)
      else if (ibasrule.eq.5) then
c case v is a HR, based on an TSB survey
       y=vin*biosurv(iy-1,2)
      else if (ibasrule.eq.6) then
      y=vin*tac(iy-1,iter)
      endif

c set the decided TAC decided with vin for the whole decision period 
      tactemp(iy)=y
      step=(tactemp(iy)-tactemp(iy-1))/float(idur)
      do  i=iy,iy+idur-1
      if (idur.gt.1.and.iabrgrad.eq.2) then
      tactemp(i)=tactemp(i-1)+step
      else
      tactemp(i)=tactemp(iy)
      endif
      enddo                 

c To find the v that this leads to, project the stock forwards with 
c unconstrained TAC according to rule (call to rule and translation)
      call project(tactemp,tacout,iy,iflg)
       
c Build basis  
      call buildbasis(iy,basis)
c call the rule
      call rule(vout,basis)
      
c translate this v into TAC      
      if (ibasrule.eq.1) then
c case v is an F-value
      call yield(rny,y,vout,1)
      else if (ibasrule.eq.2) then
c case v is a HR rule based  on an assessment
       call calcbplus(rny,tsb)
       y=vout*tsb
      else if (ibasrule.eq.3) then
c case v is a HR, based on the basis1 for decision
       y=vout*basis(1)
      else if (ibasrule.eq.4) then
c case v is a TAC rule
      y=vout
      endif

c set the decided TAC for the whole decision period
      tactemp(iy)=y
      step=(tactemp(iy)-tactemp(iy-1))/float(idur)
      do  i=iy,iy+idur-1
      if (idur.gt.1.and.iabrgrad.eq.2) then
      tactemp(i)=tactemp(i-1)+step
      else
      tactemp(i)=tactemp(iy)
      endif
      enddo                 
      end                   
      
       
      subroutine rule(v,basis)
      include 'hcscom17_1.i'
      real v, basis(mbasis)

      if (irule.eq.1) then
      call rule1(v,basis)
      else if (irule.eq.2) then
      call rule2(v,basis)
      else if (irule.eq.3) then
      call rule3(v,basis)
      else if (irule.eq.4) then
      call rule4(v,basis)
      else if (irule.eq.5) then
      call rule5(v,basis)
      else if (irule.eq.6) then
      call rule6(v,basis)
      else if (irule.eq.7) then
      call rule7(v,basis)     
      else if (irule.eq.8) then
      call rule8(v,basis)
      else
      write(*,*) 'Unknown harvest rule',irule
      stop
      endif
      end
     
      
      subroutine rule1(v,basis)
      include 'hcscom17_1.i'
c alfa rule 
      real v,basis(mbasis)
      btrig1=hcrparam(1)
      btrig2=hcrparam(2)
      vstd=hcrparam(3)
      alfa1=hcrparam(4)
      alfa2=hcrparam(5)
      vmin=hcrparam(6)
      vmax=hcrparam(7)
      v=vstd
      if (basis(1).lt.btrig1) then
        if (btrig1.gt.0.0) then
         v=vstd*(1.0-alfa1*(btrig1-basis(1))/btrig1)
        else
         v=vstd
        endif
c       if (vstd.ge.0.3) write(*,*) 'rule1',v,vstd,alfa1,btrig1,basis(1)
       else if (basis(2).gt.btrig2) then
        if (btrig2.gt.0) then
         v=vstd*(1.0+alfa2*(basis(2)-btrig2)/btrig2)
        else
         v=vstd
        endif
       endif
       if (v.lt.0.0) v=0.0
       if (v.lt.vmin) v=vmin
       if (v.gt.vmax) v=vmax
       end


      subroutine rule2(v,basis)
      include 'hcscom17_1.i'
      real v,basis(mbasis)
c slope between Btrig1 and B0, flat below
c Both b0 and btrig1 relate to basis(1)
      b0=hcrparam(1)
      btrig1=hcrparam(2)
      btrig2=hcrparam(3)
      vstd=hcrparam(4)
      vlevel1=hcrparam(5)
      alfa2=hcrparam(6)
      vmax=hcrparam(7)
      if (basis(1).lt.b0) then
      v=vlevel1
      else if (basis(1).lt.btrig1) then
      v=vlevel1+(vstd-vlevel1)*(basis(1)-b0)/(btrig1-b0)
      else if (basis(2).lt.btrig2) then
      v=vstd
      else
      v=vstd*(1.0+alfa2*(basis(2)-btrig2)/btrig2)
      endif
       if (v.lt.0.0) v=0.0
       if (v.gt.vmax) v=vmax
      end
      
      subroutine rule3(v,basis)
c Upper part of logistic function below B1 and above B2
      include 'hcscom17_1.i'
      real v,basis(mbasis)
      btrig1=hcrparam(1)
      btrig2=hcrparam(2)
      slope1=hcrparam(4)
      slope2=hcrparam(5)
      vstd=hcrparam(3)
      vmax=hcrparam(6)
      if (basis(1).lt.btrig1) then
        if (btrig1.gt.0.0) then
        vact=1.0/(1.0+exp(-slope1*(basis(1)/btrig1)))
        v0=1.0/(1.0+exp(-slope1))-0.5
        v=vstd*(vact-0.5)/v0
        else
         v=vstd
        endif
      else if (basis(2).gt.btrig2) then
        if (btrig2.gt.0) then
        vact=1.0/(1.0+exp(-slope2*(basis(2)-btrig2)/btrig2))
        v=vstd+(vact-0.5)*(vmax-vstd)
        else
         v=vstd
        endif
       if (v.lt.0.0) v=0.0
       else
       v=vstd
       endif

      end

      subroutine rule4(v,basis)
      include 'hcscom17_1.i'
c Like rule 1, but:
c vstd depends on basis(3), with a factor gain
c Basis(3) can be recruitment or a trend in something
      real v,basis(mbasis)
      btrig1=hcrparam(1)
      btrig2=hcrparam(2)
      v0=hcrparam(3)
      alfa1=hcrparam(4)
      alfa2=hcrparam(5)
      gain=hcrparam(6)/1000.0
      vact=v0+gain*basis(3)
      vmin=hcrparam(7)
      vmax=hcrparam(8)
c      write(*,*) 'in rule4',basis(1),basis(2),basis(3),btrig1,btrig2
      if (basis(1).lt.btrig1) then
        if (btrig1.gt.0.0) then
         v=vact*(1.0-alfa1*(btrig1-basis(1))/btrig1)
         else
         v=vact
        endif
c      write(*,*) 'vact1',basis(1),btrig1,vact,gain,basis(3),v
      
       else if (basis(2).gt.btrig2) then
        if (btrig2.gt.0) then
         v=vact*(1.0+alfa2*(basis(2)-btrig2)/btrig2)
        else
         v=vact
        endif
c       write(*,*) 'vact2',basis(1),btrig1,vact,gain,basis(3),v
        
        else
        v=vact
c       write(*,*) 'vact3',basis(1),btrig1,vact,gain,basis(3),v
        
       endif
        if (v.lt.vmin) v=vmin
       if (v.gt.vmax) v=vmax
c        write(*,*) 'out rule4',basis(1),btrig1,basis(2),btrig2,basis(3),
c     &gain,vact,v

       end



      subroutine rule5(v,basis)
      include 'hcscom17_1.i'
c alfa rule with two levels of vstd
      real v,basis(mbasis)
      btrig1=hcrparam(1)
      btrig2=hcrparam(2)
      refr=hcrparam(3)
      vlow=hcrparam(4)
      vhigh=hcrparam(5)
      alfa1=hcrparam(6)
      alfa2=hcrparam(7)
      vmax=hcrparam(8)
      if (basis(3).gt.refr) then
      vstd=vhigh
      else
      vstd=vlow
      endif
      v=vstd
      if (basis(1).lt.btrig1) then
        if (btrig1.gt.0.0) then
         v=vstd*(1.0-alfa1*(btrig1-basis(1))/btrig1)
        else
         v=vstd
        endif
       else if (basis(1).gt.btrig1.and.basis(2).gt.btrig2) then
        if (btrig2.gt.0) then
         v=vstd*(1.0+alfa2*(basis(2)-btrig2)/btrig2)
         else
         v=vstd
         endif
       endif
       if (v.lt.0.0) v=0.0
       if (v.gt.vmax) v=vmax
       end

      subroutine rule6(v,basis)
      include 'hcscom17_1.i'
c rule 2 with two levels of vstd
      real v,basis(mbasis)
      
      b0=hcrparam(1)
      btrig1=hcrparam(2)
      btrig2=hcrparam(3)
      refr=hcrparam(4)
      vlevel1=hcrparam(5)
      vlow=hcrparam(6)
      vhigh=hcrparam(7)      
      alfa2=hcrparam(8)
      vmax=hcrparam(9)
      if (basis(3).gt.refr) then
      vstd=vhigh
      else
      vstd=vlow
      endif
      
      v=vstd
      if (basis(1).lt.b0) then
      v=vlevel1
      else if (basis(1).lt.btrig1) then
      v=vlevel1+(vstd-vlevel1)*(basis(1)-b0)/(btrig1-b0)
      else if (basis(1).gt.btrig1.and.basis(2).gt.btrig2) then
      v=vstd*(1.0+alfa2*(basis(2)-btrig2)/btrig2)
      endif
       if (v.lt.0.0) v=0.0
       if (v.gt.vmax) v=vmax
      end

      subroutine rule7(v,basis)
c Escapement-rule
      include 'hcscom17_1.i'
      real v,basis(mbasis)    
      btrig1=hcrparam(1)
      btrig2=hcrparam(1)+hcrparam(2)
      if (basis(1).lt.btrig2) then
      v=0.0
      else
      v=basis(1)-btrig1
      endif
      end
 
       subroutine rule8(v,basis)
      include 'hcscom17_1.i'
c Data poor rule
      real v,basis(mbasis)
      btrig1=hcrparam(1)
      btrig2=hcrparam(2)
      vlow=hcrparam(3)
      vstd=hcrparam(4)
      vhigh=hcrparam(5)
      if (basis(1).lt.btrig1) then
      v=vlow
      else if (basis(1).gt.btrig1.and.basis(2).gt.btrig2) then
      v=vhigh
      else
      v=vstd
      endif
      end

      
      subroutine buildbasis(iy,basis)
c build the decision basis vector (in basis) for the TAC for year iy
c bas(year) is the intermediate value for the basis type in the year.
c Code for ibastable(basis.x); basis=1,nbasis
c     1 Type basis (1=SSB, 2=TSB, 3=assessed recruitment, 4=recruitment survey, 5=SSBsurvey, 6: B+ survey)
c     2 Time frame - first year 
c     3 Time frame last year
c     4 Calculation of basis: 1=mean over time frame, 2=minimum, 3=maximum, 4=linear trend, 5=median
      include 'hcscom17_1.i'
      real x(niter),y(niter),xx(nityr),
     &rnb(0:maxage),bas(-maxage:maxyr+mbasis),basis(1:mbasis)
      integer ibas
c Outer loop over elements in basis vector (ibas)
      do ibas=1,nbasis
      i1=ibastable(ibas,2)
      i2=ibastable(ibas,3)
      if (i2.lt.i1) then
      write(*,*) 'Warning!'
      write(*,*) 'For basis number',ibas,' the last year is before the f
     &irst year'
       stop
      endif
      
      do i=iy+i1,iy+i2
c Basis is derived from observed stocknumbers, so they are assembled first
c get the stock numbers from the right source, but only if needed
      if (ibastable(ibas,1).lt.3) then
      do ia=lowage,iaold
c use numbers from obsn if not in projection      
      if (i.lt.iy+lastassn) then
      rnb(ia)=obsn(ia,i,iy+lastassn)
      else
      rnb(ia)=projn(ia,i)
      endif
      enddo
      else if (ibastable(ibas,1).eq.3) then
      rnb(lowage)=obsn(lowage,i,iy+lastassn)
      endif

c caclulate actual basis values for year i
      if (ibastable(ibas,1).eq.1) then
      
      call calcssb(rnb,s,fobs(i),1)
      bas(i)=s
      
      else if (ibastable(ibas,1).eq.2) then
      call calcbplus(rnb,bs)
      bas(i)=bs
      else if (ibastable(ibas,1).eq.3) then
       bas(i)=rnb(lowage)
      else if (ibastable(ibas,1).eq.4) then
      bas(i)=recrsurv(i)
      else if (ibastable(ibas,1).eq.5) then
      if (biosurv(i,1).lt.0.0) then
      write(*,*) 'A biomass survey is included in the decision basis'
      write(*,*) 'in year TAC year + ',i,'which is before it has been pe
     &rformed'
      write(*,*) 'Correct the specification of the basis in the opt.inn 
     &file'
      stop
      else
      bas(i)=biosurv(i,1)
      endif
      else if (ibastable(ibas,1).eq.6) then
      if (biosurv(i,2).lt.0.0) then
      write(*,*) 'A biomass survey is included in the decision basis'
      write(*,*) 'in year TAC year + ',i,'which is before it has been pe
     &rformed'
      write(*,*) 'Correct the specification of the basis in the opt.inn 
     &file'
      stop
      else
      bas(i)=biosurv(i,2)
      endif
      endif
      enddo
c End of year loop, values for year i in bas(i)
c Calculate basis according to type     
      if (ibastable(ibas,4).eq.1) then
c mean
      s=0.0
      do j=iy+i1,iy+i2
      s=s+bas(j)
      enddo
      basis(ibas)=s/float(ibastable(ibas,3)-ibastable(ibas,2)+1)
       
      else if (ibastable(ibas,4).eq.2) then
c minimum
      s=bas(iy+i1)
      do j=iy+i1,iy+i2
      if (bas(j).lt.s) s=bas(j)
      enddo
      basis(ibas)=s

      else if (ibastable(ibas,4).eq.3) then
c maximum
      s=bas(iy+i1)
      do j=iy+i1,iy+i2
      if (bas(j).gt.s) s=bas(j)
      enddo
      basis(ibas)=s

      else if (ibastable(ibas,4).eq.4) then
c linear trend
      if (i2-i1.lt.1) then
      write(*,*) 'You ask for a trend over less than two years'
      write(*,*) 'Check definition of basis on the opt.inn file'
      stop
      endif
      do j=iy+i1,iy+i2
      jj=j-(iy+i1)+1
      x(jj)=float(jj)
      y(jj)=bas(j)
      enddo
      n=ibastable(ibas,3)-ibastable(ibas,2)+1
      call regress (x,y,a,b,n,ymean)
      if (ymean.gt.0.0) then
      basis(ibas)=b/ymean
      else
      basis(ibas)=0.0
      endif
      
      else if (ibastable(ibas,4).eq.5) then
c      write(*,*) 'median',ibastable(ibas,4)
c median
      n=i2-i1+1
      i=0
      do j=iy+i1,iy+i2
      i=i+1
      xx(i)=bas(j)
      enddo
      call sort(xx,n)
      m=int(float(n)/2.0)+1
      if (mod(n,2).eq.1) then
      basis(ibas)=xx(m)  
      else
      basis(ibas)=(xx(m-1)+xx(m))/2.0
      endif
c     write(*,*) 'obs median',n,m,xx(m)  
      
      else
      basis(ibas)=0.0
      endif
      enddo
      end



      subroutine compbasis
      
c calculate the decision basis vector
c with true Ns, for the printoutroutine printbas

      include 'hcscom17_1.i'
      real x(niter),y(niter),xx(nityr),
     &rnb(0:maxage),bas(-maxage:maxyr+mbasis),basis(1:mbasis)
      integer ibas
c Outer loop over elements in basis vector (ibas)
      do ibas=1,nbasis
      i1=ibastable(ibas,2)
      i2=ibastable(ibas,3)
      if (i2.lt.i1) then
      write(*,*) 'Warning!'
      write(*,*) 'For basis number',ibas,' the last year is before the f
     &irst year'
       stop
      endif
      do iy=1,maxyr
      do i=iy+i1,iy+i2
      
c Basis is derived from true stocknumbers, so they are assembled first
c get the stock numbers from the right source, but only if needed
      if (ibastable(ibas,1).le.3) then
      do ia=lowage,iaold
      rnb(ia)=truen(ia,i)
      enddo
      endif

c caclulate actual basis values for year i
      if (ibastable(ibas,1).eq.1) then
      
      call calcssb(rnb,s,fobs(i),1)
      bas(i)=s
      
      else if (ibastable(ibas,1).eq.2) then
      call calcbplus(rnb,bs)
      bas(i)=bs
      else if (ibastable(ibas,1).eq.3) then
       bas(i)=rnb(lowage)
c Note that true and perceived survey results is not very meaningful.
c You get the same for true and observed here.       
      else if (ibastable(ibas,1).eq.4) then
      bas(i)=recrsurv(i)
      else if (ibastable(ibas,1).eq.5) then
      if (biosurv(i,1).lt.0.0) then
      write(*,*) 'A biomass survey is included in the decision basis'
      write(*,*) 'in year TAC year + ',i,'which is before it has been pe
     &rformed'
      write(*,*) 'Correct the specification of the basis in the opt.inn 
     &file'
      stop
      else
      bas(i)=biosurv(i,1)
      endif
      else if (ibastable(ibas,1).eq.6) then
      if (biosurv(i,2).lt.0.0) then
      write(*,*) 'A biomass survey is included in the decision basis'
      write(*,*) 'in year TAC year + ',i,'which is before it has been pe
     &rformed'
      write(*,*) 'Correct the specification of the basis in the opt.inn 
     &file'
      stop
      else
      bas(i)=biosurv(i,2)
      endif
      endif
      enddo
c End of year loop, values for year i in bas(i)
c Calculate basis according to type     
      if (ibastable(ibas,4).eq.1) then
c mean
      s=0.0
      do j=iy+i1,iy+i2
      s=s+bas(j)
      enddo
      basis(ibas)=s/float(ibastable(ibas,3)-ibastable(ibas,2)+1)
       
      else if (ibastable(ibas,4).eq.2) then
c minimum
      s=bas(iy+i1)
      do j=iy+i1,iy+i2
      if (bas(j).lt.s) s=bas(j)
      enddo
      basis(ibas)=s

      else if (ibastable(ibas,4).eq.3) then
c maximum
      s=bas(iy+i1)
      do j=iy+i1,iy+i2
      if (bas(j).gt.s) s=bas(j)
      enddo
      basis(ibas)=s

      else if (ibastable(ibas,4).eq.4) then
c linear trend
      if (i2-i1.lt.1) then
      write(*,*) 'You ask for a trend over less than two years'
      write(*,*) 'Check definition of basis on the opt.inn file'
      stop
      endif
      do j=iy+i1,iy+i2
      jj=j-(iy+i1)+1
      x(jj)=float(jj)
      y(jj)=bas(j)
      enddo
      n=ibastable(ibas,3)-ibastable(ibas,2)+1
      call regress (x,y,a,b,n,ymean)
      if (ymean.gt.0.0) then
      basis(ibas)=b/ymean
      else
      basis(ibas)=0.0
      endif
      
      else if (ibastable(ibas,4).eq.5) then
c median
      n=i2-i1+1
      i=0
      do j=iy+i1,iy+i2
      i=i+1
      xx(i)=bas(j)
      enddo
      call sort(xx,n)
      m=int(float(n)/2.0)+1
      if (mod(n,2).eq.1) then
      basis(ibas)=xx(m)  
      else
      basis(ibas)=(xx(m-1)+xx(m))/2.0
      endif
      else
      basis(ibas)=0.0
      endif
      truebas(ibas,iy)=basis(ibas)
c      write(*,*) 'truebas',iy,ibas,(xx(jj),jj=1,3),basis(ibas)
      enddo
      enddo
      end



      
      subroutine buildcbasis(cbasis,icnr,iy)
c build the decision basis vector for the constraint on TAC for year iy,
c icnr=1 is for the filter rule, icnr=2 is for the percentage rule
c cbasis is a single number, the constrint rules do not operate with a vector of bases
      include 'hcscom17_1.i'
      real cbasis,x(niter),y(niter),rnb(0:maxage),
     &bas(-maxage:maxyr+mbasis)
c find sequence of ssb and tsb
      i1=ibastable(icnr,2)
      i2=ibastable(icnr,3)
      do i=iy+i1,iy+i2
c get the stock numbers from the right source, if needed
       if (icbastable(icnr,1).le.3) then
       do ia=lowage,iaold
       if (i.lt.iy+lastassn) then
        rnb(ia)=obsn(ia,i,iy+lastassn)
       else
        rnb(ia)=projn(ia,i)
       endif
       enddo
       endif
c then we can calculate the basis for year i       
       if (icbastable(icnr,1).eq.1) then
        call calcssb(rnb,s,fobs(i),1)
        bas(i)=s
       else if (icbastable(icnr,1).eq.2) then
        call calcbplus(rnb,bs)
        bas(i)=bs
       else if (icbastable(icnr,1).eq.3) then
        bas(i)=rnb(lowage)
       else if (icbastable(icnr,1).eq.4) then
        bas(i)=recrsurv(i)
       else if (icbastable(icnr,1).eq.5) then
        if (biosurv(i,1).lt.0.0) then
         write(*,*) 'A biomass survey is included in the decision basis'
      write(*,*) 'in year TAC year + ',i,'which is before it has been pe
     &rformed'
      write(*,*) 'Correct the specification of the basis in the opt.inn 
     &file'
         stop
        else
         bas(i)=biosurv(i,1)
        endif
       else if (icbastable(icnr,1).eq.6) then
        if (biosurv(i,2).lt.0.0) then
         write(*,*) 'A biomass survey is included in the decision basis'
      write(*,*) 'in year TAC year + ',i,'which is before it has been pe
     &rformed'
      write(*,*) 'Correct the specification of the basis in the opt.inn 
     &file'
         stop
        else
         bas(i)=biosurv(i,2)
        endif
       endif
      enddo
c end year loop, bas(i) has the basis for year i
 
c Calculate basis according to type      
      if (icbastable(icnr,4).eq.1) then
c mean
      s=0.0
      do j=iy+i1,iy+i2
      s=s+bas(j)
      enddo
      cbasis=s/float(icbastable(icnr,3)-icbastable(icnr,2)+1)
      else if (icbastable(icnr,4).eq.2) then
c minimum
      s=bas(iy+i1)
      do j=iy+i1,iy+i2
      if (bas(j).lt.s) s=bas(j)
      enddo
      cbasis=s
      else if (icbastable(icnr,4).eq.3) then
c linear trend
      if (i2-i1.lt.1) then
      write(*,*) 'You ask for a trend over less than two years'
      write(*,*) 'Check definition of basis on the opt.inn file'
      stop
      endif
      do j=iy+i1,iy+i2
      jj=j-(iy+i1)+1
      x(jj)=float(jj)
      y(jj)=bas(j)
      enddo
      n=icbastable(icnr,3)-icbastable(icnr,2)+1
      call regress (x,y,a,b,n,ymean)
      if (ymean.gt.0.0) then
c Absolute trend in this case
      cbasis=abs(b/ymean)
      else
      cbasis=0.0
      endif
      endif
      end


      subroutine prepproj(iy)
      include 'hcscom17_1.i'
c Numbers in projn from obsn, up to and including iy+lastassn
c Fill in assumed values for prerecruits if neded
      integer iy
      do i=iy+lastassn+iobsstart,iy+lastassn
      do ia=0,iaold
      projn(ia,i)=obsn(ia,i,iy+lastassn)
      enddo
c      write(*,*) 'preproj',iy,i,(projn(ia,i),ia=0,3)
      enddo
c substitute for year classes not recruited
c have recmean ready. Done here, presuming that they are never needed 
c when no projection is needed
      if (ioptdr1.eq.1.or.ioptdr2.eq.1) call getmeanrecr(iy)

      if (lowage.gt.0) then
c calculate initial recruits      
      if (ioptdr1.eq.1) then
c case mean values      
      do ia=0,lowage
      projn(ia,iy+lastassn)=recmean
      enddo
c      write(*,*) 'recmean',iy,recmean
c      write(*,*) 'preproj2',iy,i,(projn(ia,iy+lastassn),ia=0,3)
      else if (ioptdr1.eq.2) then
c case random values
      do ia=0,lowage
      x=rnx(rintvar)
      xsi=snrn(x)
      xsi=xsi*sigmadr
      projn(ia,iy+lastassn)=projn(ia,iy+lastassn)*exp(xsi)
      enddo
      endif   
      endif
      end      

      subroutine project(tacinn,
     &tacout,iy,iflg)

c Deterministic projection presumed to start in year iy+lastassn and continuing to iy+iproj      
c Assumed TACs are tacinn. 
c Tacout holds TACs that come from the findf subroutine
c It will deviate from tacinn if the tac in tacinn cannot be taken
c Assumes projn filled in up to and including iy+lastassnc Recruitments are according to the deterministic SR function.
c The results of the projection is in the common projn
c  
      include 'hcscom17_1.i'
      real rn1(0:maxage),
     &tacinn(0:maxyrp),tacout(0:maxyrp)
      integer iy,iflg,iflg1
      iflg=0
c start with numbers from last assessment in projn (done by prepproj)    
      call prepproj(iy)   
      do ia=0,iaold
      rn1(ia)=projn(ia,iy+lastassn)
      enddo
c      write(*,*) 'in proj     ',iy+lastassn,(rn1(ij),ij=0,3)
c and we start with tacout=tacinn      
      tacout(iy)=tacinn(iy)
      
            
      if (iproj.gt.0.or.lastassn.lt.0) then
c projection relevant      
       do 20 i=iy+lastassn,iy+iproj
       if (i.le.(iy+idur-1)) then
c i is a year with at least preliminary TAC      
        tacout(i)=tacinn(i)
c The F used in the projection is derived here from the TAC
c The is no way to apply the average 'f-sq'.        
        call findf(rn1,tacout(i),f,iflg1,1)

        if (iflg1.eq.1) iflg=1
       else
c i is a year after the last TAC. Assume same f      
        call yield(rn1,y,f,1)
        tacout(i)=y
       endif
c       if (i.le.maxyr) fobs(i)=f
       call calcssb(rn1,s,f,1)
       sr=0.0
       if (i.lt.iy+iproj) then
        call nextyear(rn1,f,s,1,ispdelay,-1)
c        write(*,*) 'in proj next',i+1,(rn1(ii),ii=0,3),f,s

        do ia=0,iaold
         projn(ia,i+1)=rn1(ia)
        enddo

c Substitute future recruitments        
        if (ioptdr2.eq.1) then
c Case future recruitments are deterministic according to SSB in the 
c current year from nextyear
        r0=recmean
        else if (ioptdr2.eq.2) then
c case future recruitmennts from SR function with delay = iydrec3,
c but with s from nextyear   
        iyrec=iy+lastassn+iydrec3  
        if (iyrec.lt.1) iyrec=1
        call recruitment(r0,iyrec,s,0)
        endif
        do iia=0,lowage
        projn(iia,i+1)=r0
        enddo
       endif
c        write(*,*) 'r0',r0
c        write(*,*) 'in proj fine',i+1,(rn1(ii),ii=0,3),f,s

20    continue

      endif
 
      end      
            
            
      subroutine getmeanrecr(iy)
c Get the recruitment in projection as the mean of percieved recruitments over n years      
      include 'hcscom17_1.i'
      integer iy
      recmean=0.0
      do i= iy+lastassn+iydrec1,iy+lastassn+iydrec2
      recmean=recmean+log(obsn(lowage,i,iy+lastassn))
      enddo
      recmean=exp(recmean/(float(iydrec2-iydrec1+1)))

      end
     
     
      subroutine obsmodel(rnold,iy,iback)
c converts the real stock numbers to corrupted stock numbers
c Input is current true rn at age in rnold, output in common obsn 
c This is the history as percieved in year iy, backcalculated from the start of iy.
c In addition, it updates the correspondning F-values representing the last assessment
c It also holds the newly drawn xsi-values in xsiobs, that are used
c for calculating the xsi to be applied as a autoregressive process
      include 'hcscom17_1.i'
      real rnold(0:maxage)
      integer iy,iback
      biasobs= hcrparam(17)
      yfctobs=hcrparam(18)
      acfactobs=hcrparam(19)
c build a new vector of random values.
      x=rnx(rintvar)
      xsi=snrn(x)*yfctobs
c include autocorrelation (acfactobs and xsiycftobs(iy)     are commons
      xsi=acfactobs*xsiycftobs(iy-1)+xsi*sqrt(1.0-acfactobs**2)
      xsiycftobs(iy)=xsi
      btemp=(1.0+biasobs)*exp(xsi-yfctobs**2/2.0)
      
c  By age
      do ia=0,iaold
      if (cvobs(ia,1).gt.0.0) then
      x=rnx(rintvar)
      xsi=snrn(x)
      xsi=xsi*cvobs(ia,1)
      else
      xsi=0.0
      endif
      xsiobs(ia,iy)=btemp*exp(xsi-cvobs(ia,1)**2/2.0)
      enddo
c get the actual xsi-value for the current year
      do ia=0,iaold
      a=0.0
      xsiact=0.0
      do i=0,nac-1
      if (ia-i.ge.lowage) then
      xsiact=xsiact+acfact(i)*xsiobs(ia-i,iy-i)
      a=a+acfact(i)
      endif
      enddo
      if (a.gt.0.0) then
      xsiact=xsiact/a
      else
      xsiact=1.0
      endif
c Apply to n-values in the last year
      obsn(ia,iy,iy)=rnold(ia)*xsiact
c obsn is copied to projn to always have the numbers there even if 
c there is no projection. 
      projn(ia,iy)=obsn(ia,iy,iy)
      enddo
c      write(*,*) 'truen  ',iy,(rnold(ia),ia=0,3)
c      write(*,*) 'in obsn',iy,(obsn(ia,iy,iy),ia=0,3)

      if (iy.eq.0) then
      endif
      if (iback.gt.0) call vpaobs(iy)
c      write(*,*) 'postVPA',iy,(obsn(ia,iy,iy),ia=0,3)
      
      
      end
      
      
      
      subroutine vpaobs(iyref)
      include 'hcscom17_1.i'
       
c VPA backwards until iy+iobsstart (seen from year iyref)
c Note that iobsstart typically is negative
c Fills the local table rntable, transfers it to obsn (age,year,iyref) at the end
c The correponding Fs are not kept, fobs is generated in bootruns when
c deciding on bank-borrow and not updated.

      real rntable(0:maxage,-maxage:maxyr+mbasis),
     &ftable(0:maxage,-maxage:maxyr+mbasis)
      integer iyref
    
c fill in terminal n-values      
      do ia=0,maxage
      do iy=iobsstart,iyref
      rntable(ia,iy)=-1.0
      ftable(ia,iy)=-1.0
      enddo      
      rntable(ia,iyref)=obsn(ia,iyref,iyref)    
      enddo
c cohorts that have terminal n's (Popes eqn)
      do iycl=1,iaold-1
c iycl is the year class, numbered as age in terminal year
      do i=1,iycl
      ia=iycl-i
      iy=iyref-i
      if (ia.ge.0.and.iy.ge.iobsstart) then
      rntable(ia,iy)=rntable(ia+1,iy+1)*exp(rm(ia))+cobs(ia,iy)*
     &exp(rm(ia)/2.0)
      if (rntable(ia,iy).gt.0.0.and.rntable(ia+1,iy+1).gt.0.0) then
      ftable(ia,iy)=log(rntable(ia,iy)/rntable(ia+1,iy+1))-rm(ia)
      
      else
c Exception: One of the n's is 0, presumabluy the last one. 
c F sahould in principle be infinity, set it at fmax.    
      ftable(ia,iy)=fmax
      endif
      endif
      enddo
      enddo
       
c cohorts starting at oldest true age
      iaot=iaold-1
c N at oldest true age ia in year j < iyref    
      do j=iyref-1,iyref+iobsstart-1,-1  
      ftable(iaot,j)=ftable(iaot-1,j)
      z=ftable(iaot,j)+rm(iaot)
      if (ftable(iaot,j).gt.0.0) then
      rntable(iaot,j)=cobs(iaot,j)*z/ftable(iaot,j)/(1.0-exp(-z))
      else
c exception handling if there is a catch, but the assumed F, taken from
c the year before was 0
c Not sophisticated, just assume N = 0, i.e. the year class was empty
c at the last true age
      rntable(iaot,j)=0.0
      endif
      
c calculate that cohort backwards    
      do i=1,-iobsstart
      ianow=iaot-i
      iynow=j-i
      if (iynow.ge.iyref+iobsstart.and.ianow.ge.0) then
      rntable(ianow,iynow)=rntable(ianow+1,iynow+1)*exp(rm(ia))
     &+cobs(ianow,iy)*exp(rm(ianow)/2.0)
        
      if (rntable(ianow,iynow).gt.0.0.and.
     &rntable(ianow+1,iynow+1).gt.0.0) then
      ftable(ianow,iynow)=log(rntable(ianow,iynow)/
     &rntable(ianow+1,iynow+1))-rm(ia) 
      if (ftable(ianow,iynow).lt.0.0) ftable(ianow,iynow)=fmax      
      else
      ftable(ianow,iynow)=fmax
      endif
      endif
      enddo
      

c calculate the plus group
      if (ftable(iaot-1,j).gt.0.0) then
      ftable(iaot,j)=ftable(iaot-1,j)
      z=ftable(iaot,j)+rm(iaot)
      rntable(iaold,j)=cobs(iaold,j)*z/ftable(iaot,j)/(1.0-exp(-z))
      else
      rntable(iaold,j)=0.0
      endif
      enddo

c transfer to obsn
      do ia=0,maxage
      do iy=iyref+iobsstart,iyref
      obsn(ia,iy,iyref)=rntable(ia,iy)
      enddo
      enddo
       
      end


      subroutine getobscanum(canum,iy)
      include 'hcscom17_1.i'
      real canum(0:maxage)
      integer iy
      do ia=0,maxage
      cobs(ia,iy)=0.0
      enddo
      cv=hcrparam(20)
      do ia=lowage,iaold
      x=rnx(rintvar)
      xsi=snrn(x)
      xsi=xsi*cv
      cobs(ia,iy)=canum(ia)*exp(xsi-cv/2.0)
      enddo
      end


      subroutine getrecrsurvey(iy,r)
c Imitate a recruitment survey, based on actual recruitment, with 
c catchability q which acts as a bias.
c Called together with obsmodel. Result for the year iy is available
c at decison time
      include 'hcscom17_1.i'
      real r,cv
      integer iy
      q=hcrparam(21)
      cv=hcrparam(22)
      x=rnx(rintvar)
      xsi=snrn(x)
      xsi=xsi*cv
      recrsurv(iy)=r*q*exp(xsi-cv/2.0)
      end


      subroutine getobssurvey(iy,rn,fact,isel)
c Imitate a biomass survey,by corrupting true biomass.
c The catchability q acts as a bias here
c Called together with obsmodel. Result for the year iy is available
c at decison time
      include 'hcscom17_1.i'
      real rn(0:maxage),fact,cv,s,bs
      integer iy,isel
      q = hcrparam(23)
      cv=hcrparam(24)
      call calcssb(rn,s,fact,isel)
      x=rnx(rintvar)
      xsi=snrn(x)
      xsi=xsi*cv
      biosurv(iy,1)=s*q*exp(xsi)

      call calcbplus(rn,bs)
      x=rnx(rintvar)
      xsi=snrn(x)
      xsi=xsi*cv
      biosurv(iy,2)=bs*q*exp(xsi-cv/2.0)
      end


      subroutine implement(iy,yc,yci,rn,jflg,canum)
      include 'hcscom17_1.i'
c Adds implemetation error to the presumed catch in numbers.
c calculates the true f at age and common f
c Input: yc: Agreeed TAC
c        rn: True stock numbers
c Output: yci: realized catch
c         f: true f-factor
c         canum: actual removal at age
c Revised in version 12_3     
      real yc,yci,f,canum(0:maxage),rn(0:maxage),fa(0:maxage)
      integer iy,isel,jflg
      f=truef(iy,iter)
c isel: 1: Standard selection, 2. true selection after implementation)
      biasimp= hcrparam(25)
      yfctimp= hcrparam(26)
c Transform TAC to canum assuming standard selection:
      isel=1
      jflg=0
      call findf(rn,yc,ftemp,jflg,isel)
      do 11 ia=0,iaold
      if (ia.lt.lowage) then
      canum(ia)=0.0
      else
      z=ftemp*sel(ia,isel)+rm(ia)
      if (z.gt.0.0) then
      canum(ia)=ftemp*sel(ia,isel)*rn(ia)*(1.0-exp(-z))/z
      else
      canum(ia)=0.0
      endif
      endif
11    continue
c apply noise to canum
c find the yearfactor common to all ages (biased random)
c gives yci: which shall be realized SOP
      x=rnx(rintvar)
      xsi=snrn(x)
      xsi=xsi*yfctimp
      btemp=(1.0+biasimp)*exp(xsi)
      yci=yc*btemp
c find age factor for each age and apply, gives noisy and biased canum      
      do 10 ia=lowage,iaold
      if (cvimp(ia).gt.0.0) then
      x=rnx(rintvar)
      xsi=snrn(x)
      xsi=xsi*cvimp(ia)
      else
      xsi=0.0
      endif
      canum(ia)=canum(ia)*exp(xsi)*btemp
      if (canum(ia).le.0.0) canum(ia)=0.0
 
10    continue
c translate canum to yield (SOP)      
      yctemp=0.0
      do 23 ia=lowage,iaold
      yctemp=yctemp+canum(ia)*weca(ia)
23    continue
c adjust canum to get the right realized SOP
      do ia=lowage,iaold
      if (yctemp.gt.0.0) then
      canum(ia)=canum(ia)*yci/yctemp
      else
      canum(ia)=0.0
      endif
      enddo
c Find true F corresponding to canum at each age
      do 20 ia=lowage,iaold
      if (canum(ia).le.0.0) then
      fa(ia)=0.0
      else
       ef=1.0-canum(ia)/rn(ia)*exp(rm(ia)/2.0)
c protection if the stock is not sufficient for the catch at an individual age
       if (ef.gt.0.0) then
       fa(ia)=-log(ef)
       else
       fa(ia)=fmax
       jflg=1
       endif
      endif
20    continue
c Scale fa to get realized selection (isel=2)
      f=0.0
      do 22 ia=iaref1,iaref2
      f=f+fa(ia)
22    continue
      f=f/float(iaref2-iaref1+1)
      do 21 ia=lowage,iaold
      if (f.gt.0.0) then
      sel(ia,2)=fa(ia)/f
      else
      sel(ia,2)=0.0
      endif
21    continue 
c get true F level giving the true sop with this selection
      call findf(rn,yci,f,jflg,2)
      truef(iy,iter)=f
      end      
      
      subroutine findf(rn,yy,fact,jflg,isel)
c find f-multiplier=fact corresponding to yield yy  
c If that is not achievable within f = fmax, 
c fact=fmax and yy=yield corresponding to fmax is returned,
c and jflg is set    
      include 'hcscom17_1.i'

c local variables      

      real rn(0:maxage),fact,delta,yy,y1,y2
      integer jflg,isel
      jflg=0
      if (yy.lt.0.0001) then
      fact=0.0
      goto 99
      endif
c if the catch cannot be obtained by F=fmax,
c set jflg, return the catch corresponding to fmax
c and leave    
c      write(*,*) 'Enter yield',yy,fmax,rn
      call yield(rn,y1,fmax,isel)
c      write(*,*) 'Yield fmax',y1,yy,rn

      if (y1.lt.yy) then
      jflg=1
      fact=fmax
      goto 99
      endif
c Start searching
      delta=0.5
      jflg=0
      fact=0.5
      call yield(rn,y1,fact,isel)
      
10    if (y1.gt.yy) then
c current F too high (y1>yy)
       if (fact.lt.0.001) then
        fact=0.0
        yy=0.0
        jflg=2
        goto 99
       endif
       fact=fact*(1.0-delta)
      else

       fact=fact*(1.0+delta)
      endif

      y2=y1
      call yield(rn,y1,fact,isel)
      if ((y1-yy)*(y2-yy).le.0.0) then
c reduce delta if straddling over the target
       delta=delta*0.6
       if (delta.lt.0.00001) goto 99
      endif
      goto 10
      
99    if (jflg.gt.0) call yield(rn,yy,fact,isel)
      end
 

      subroutine yield(rn,y,fact,isel)
c computes the yield (= y)from stock numbers and f-factor (=ff)  
c and the current weights  
      include 'hcscom17_1.i'
c local variables
      real fact,y,rn(0:maxage)  
      integer isel   
      y=0.0
      do 22 ia=0,iaold
      z=rm(ia)+sel(ia,isel)*fact
      if (z.gt.0.0) then
      y=y+rn(ia)*sel(ia,isel)*fact*(1.0-exp(-z))/z*weca(ia)   
      else
      y=0.0
      endif
22    continue  
      end

      subroutine yielda(rn,y,cn,fact,agecatch,isel)
c computes the yield (= y)from stock numbers and f-factor (=ff)  
c and also the mean age in the catch, for use in the yieldrecr subroutine  
      include 'hcscom17_1.i'
c local variables
      real fact,y,rn(0:maxage),agecatch,cn(0:maxage)  
      integer isel   
      y=0.0
      ay=0.0
      do 22 ia=0,iaold
      z=rm(ia)+sel(ia,isel)*fact
      if (z.gt.0.0) then
      cn(ia)=rn(ia)*sel(ia,isel)*fact*(1.0-exp(-z))/z
      y=y+cn(ia)*weca(ia) 
      ay=ay+float(ia)*cn(ia)*weca(ia)   
      else
      y=0.0
      cn(ia)=0.0
      endif
22    continue
      if (y.gt.0.0) then
      agecatch=ay/y     
      else
      agecatch=-1.0
      endif
      end

      
      subroutine calcssb(rn,s,fact,isel)
      include 'hcscom17_1.i'
c local variables
      real rn(0:maxage),s
      integer isel
      s=0.0
      do 22 ia=lowage,iaold
      s=s+rn(ia)*exp(-sel(ia,isel)*fact*propf-rm(ia)*propm)
     &*west(ia)*rmat(ia)  
c      write(*,*) 'calcssb',ia,isel,fact,rm(ia),sel(ia,isel),west(ia),
c     &rmat(ia),s
22    continue     
      end

      subroutine calcssba(rn,s,sn,fact,agessb,isel)
c calculates the mean age at spawning in addition to the SSB
c For use in the yield/recruit routine
c Timing is dedtermined by propm
      include 'hcscom17_1.i'
c local variables
      real rn(0:maxage),sn(0:maxage),s,fact,agessb
      integer isel
      s=0.0
      sa=0.0
      do 22 ia=lowage,iaold
      sn(ia)=rn(ia)*exp(-sel(ia,isel)*fact*propf-rm(ia)*propm)*rmat(ia)
      s=s+sn(ia)*west(ia)
      sa=sa+float(ia)*sn(ia)*west(ia)  
22    continue 
      agessb=sa/s    
      end      

      subroutine calctsb(rn,tsb)
      include 'hcscom17_1.i'
      real rn(0:maxage),tsb
      tsb=0.0
      do 10 ia=itsbage,iaold
      tsb=tsb+rn(ia)*west(ia)
10    continue
      end    
 
      subroutine calcbplus(rn,bs)
c Note: The B+ biomass in the HR rule uses catch weights!
      include 'hcscom17_1.i'
      real rn(0:maxage),bs
      bs=0.0
      do 10 ia=ibplusage,iaold
      bs=bs+rn(ia)*weca(ia)
10    continue
      end      

      subroutine updatewema(iy,rn)
c Updates the components of weights and maturities that do
c not depend on age: 
c pfactor: Periodic factor
c Randfactor: Year component of random noise
c yclfact: year class factor 
c Calculates TSB using weights from year before

c Then, calls calcwema, that calculates the weightrs at age
c Called before the update of the stocknumbers in nextyear
      include 'hcscom17_1.i'
      real rn(0:maxage)
      integer iy
      call calctsb(rn,tsb)      
c Periodic trend - year factor
      if (wperiod.le.0.001) then
      pfactor=1.0
      else
      pfactor=1.0+wampl*cos(6.283185*(float(iy)-wphase)/
     &wperiod)  
      endif
c random factor for the year
      x=rnx(rintvar)
      xsi=snrn(x)
      xm=1.0+xsi*wsigma 
      randfactory=xm 
c Age loop
      do ia=lowage,iaold
c year class factor, originally generated in subroutine recr
      if (iy.ge.-iaold) then
      yclfct=wyclfct(iy-ia)
      else
      yclfct=1.0
      endif
      call calcwema(ia,tsb,pfactor,randfactory,yclfct,1)
      trueweca(iy+1,ia)=weca(ia)
      truewest(iy+1,ia)=west(ia)
      truemat(iy+1,ia)=rmat(ia)
      enddo
      end
 
      subroutine calcwema
     &(ia,tsb,pfactor,randfactory,yclfct,kflg)
      include 'hcscom17_1.i'
      integer ia,kflg
c Note: iy in the call is only used in the increment option, 
c that is commented out      

c kflg=0 means that no age-dependent random term (for use in Yieldrecr)
c else 1
      real tsb,pfactor,randfactory,yclfct
c Density dependence and mulitpliers (year class factor is generated in recruit)
c when the year class is born
c Stock weights
      if (kflg.ne.0) then
c random age factor - other factors from updatewema     
      x=rnx(rintvar)
      xsi=snrn(x)
      xm=1.0+xsi*westspec(ia,5)
      randfactora=xm 
      else
      randfactora=1.0
      endif

c wtemp is the TSB dependent component      
c Old alternative 1: Modify weights      
      wtemp=westspec(ia,1)
c New alternative: Modify weight increments
c wincralfa=0 gives almost, but not quite, the old version
c      wincrfact=(1.0+wincralfa*(wbref-tsb)/wbref)
c      if (wincrfact.lt.wincrlow) wincrfact=wincrlow
c      if (wincrfact.gt.wincrhigh) wincrfact=wincrhigh
c      wtemp=
c     &truewest(iy,ia-1)+(westspec(ia,1)-westspec(ia-1,1))*wincrfact
c  Modify wtemp
      west(ia) = wtemp*(1.0+westspec(ia,4)*(wbref-tsb)/wbref)*
     &pfactor*randfactory*randfactora*yclfct     
c Catch weights      
      if (kflg.ne.0) then
c random age factor - other factors from updatewema     
      x=rnx(rintvar)
      xsi=snrn(x)
      xm=1.0+xsi*wecaspec(ia,5)
      randfactora=xm 
      else
      randfactora=1.0
      endif
c wtemp is the TSB dependent component      
c Old alternative 1: Modify weights      
      wtemp=wecaspec(ia,1)*(1.0+wecaspec(ia,4)*(wbref-tsb)/wbref)
c New alternative: Modify weight increments on top of TSB effect on weight at age
c wincralfa=0 gives almost, but not quite, the old version
c      wincrfact=(1.0+wincralfa*(wbref-tsb)/wbref)
c      if (wincrfact.lt.wincrlow) wincrfact=wincrlow
c      if (wincrfact.gt.wincrhigh) wincrfact=wincrhigh
c      wtemp=
c     &trueweca(iy,ia-1)+(wecaspec(ia,1)-wecaspec(ia-1,1))*wincrfact
c     &*(1.0+wecaspec(ia,4)*(wbref-tsb)/wbref)

c  Modify wtemp
      weca(ia) = wtemp*pfactor*randfactory*randfactora*yclfct     
c      write(*,*) iy,ia,wtemp,weca(ia)
     
c Density dependence of maturation is by referring to an age agemat that 
c is different from the actual age
      if (kflg.ne.0) then
c random age factor - other factors from updatewema     
      x=rnx(rintvar)
      xsi=snrn(x)
      xm=1.0+xsi*voksenspec(ia,5)
      randfactora=xm 
      else
      randfactora=1.0
      endif
      agemat = float(ia)*(1.0+voksenspec(ia,4)*(wbref-tsb)/wbref)*
     &pfactor*randfactory*randfactora*yclfct
c Find maturation corresponding to age = agemat
      i1=int(agemat)
      i2=i1+1
      if (i1.lt.lowage) then
      rmat(ia)= voksenspec(lowage,1)
      else if (i2.gt.iaold) then
      rmat(ia)=voksenspec(iaold,1)
      else
      rmat(ia)=voksenspec(i1,1)+(voksenspec(i2,1)-voksenspec(i1,1))*
     &(agemat-float(i1))
      endif
 
c Outer bounds
      if (weca(ia).lt.wecaspec(ia,2)) weca(ia)=wecaspec(ia,2)
      if (weca(ia).gt.wecaspec(ia,3)) weca(ia)=wecaspec(ia,3)
      if (west(ia).lt.westspec(ia,2)) west(ia)=westspec(ia,2)
      if (west(ia).gt.westspec(ia,3)) west(ia)=westspec(ia,3)
      if (rmat(ia).lt.voksenspec(ia,2)) rmat(ia)=voksenspec(ia,2)
      if (rmat(ia).gt.voksenspec(ia,3)) rmat(ia)=voksenspec(ia,3)
c extra security on rmat
      if (rmat(ia).gt.1.0) rmat(ia)=1.0
      if (rmat(ia).lt.0.0) rmat(ia)=0.0

      end
      
      subroutine nextyear(rn,fact,s,isel,idel,iy)
c Project the stock forward for one year, according to
c a fishing mortality fact.
c Draw and include a new year class
c The stock is represented in rn, which is updated
c If the flag idel =0, a provisional SSB is calculated for the S-R function
c else S is used for SSB.
c The provisional SSB is returned in s
c iy is only used when the true stock is updated, and only to include 
c autocorrelation. Else it is -1
      include 'hcscom17_1.i'
c local variables
      real rn(0:maxage),fact,s
      integer isel,idel,iy
      do 20 ia=iaold-2,0,-1
      rn(ia+1)=rn(ia)*exp(-rm(ia)-sel(ia,isel)*fact)
20    continue
c plus gp
      rn(iaold)=rn(iaold)*exp(-rm(iaold)-sel(iaold,isel)*fact)+ 
     &rn(iaold-1)*exp(-rm(iaold-1)-sel(iaold-1,isel)*fact)  
c set rn(0) to 0 until a recruitment has been entered      
      rn(0)=0.0
c get provisional SSB if needed, used in recruitment)
      if (idel.eq.0) call calcssb(rn,s,fact,isel)
      call recruitment(rcr,iy,s,1)
      rn(0)=rcr
      end
      


      subroutine recruitment(rcr,iy,s,iflg)
c generates a recruitment rcr according to the the specified function with
c SSB = function of s * multipliers in common rmult
c Parameters (fixed):
c a-parameter, called asrparam (common)
c b-parameter, called bsrparam (common) 

c iflg=0 gives the deterministic recruitment, iflg =1 gives the stochastic.
c Functions (jfunc: common) - generates deterministic values r1
c 1: Hockey stick r1=a if s>b; else r=a*s/b
c 2: Beverton - Holt r1=a*s/(b+s)
c 3: Ricker r1=a*s*exp(-b*s)


      include 'hcscom17_1.i'
c local variables
      real rcr,s
      integer iy,iflg
      ireg=nregact(iy,iter)
      asrparam=recrparam(ireg,iter,1)   
      bsrparam=recrparam(ireg,iter,2)   
      jfunc=irecrcode(ireg,iter,1)
c find mean recruitment r1
      if (jfunc.lt.3) then
       r1=asrparam
      else if (jfunc.eq.3) then
       r1=asrparam*s/bsrparam*exp(1.0-s/bsrparam)
      else
       write(*,*) 'Incomprehensible recruit function', jfunc
      endif
c Final nummber
      if (iflg.eq.0) then
      rcr=r1
      else
      rcr=r1*rmult(iy,1)*rmult(iy,2)*rmult(iy,3)
      endif

c calculate the year class factor for weights and maturities
c This can always be calculated, but is only to be used 
c when recruit is used to make a new real year class
      wyclfct0=1.0+wyclalfa*(wrref-rcr)/wrref
      if (wyclfct0.lt.wycllow) wyclfct0=wycllow
      if (wyclfct0.gt.wyclhigh) wyclfct0=wyclhigh
c reset of the recruitment parameters to originals, to avoid
c getting stuck with wrong values in the wrong places
      asrparam=asrorg
      end

      subroutine yieldrecr
c Stand-alone routine to calculate yield and SSB per recruit
c at equilibrium between F=0 and Fcrash
c If there is spasmodic recruitment, it is assumed that the a-parameter
c reflects the basic recruitment. The recruitment level
c corresponds to the average recruitment, taking the spasmodic
c multiplier into account.
c
c If there is more than one recruitment regime, the routine here is repeated
c for each regime.
      include 'hcscom17_1.i'
      real rn(0:maxage),cn(0:maxage),sn(0:maxage)
c Set up population properties and initialise arrays
c If properites vary between iterations we use iteration 1.
      iter=1
      do ireg=1,nregimes
c transfer current recruitment parameters to working variables
      asrorg=recrparam(ireg,iter,1)   
      asrparam=recrparam(ireg,iter,1)   
      bsrparam=recrparam(ireg,iter,2)   
      sigmar=recrparam(ireg,iter,3)   
      rtrunc1=recrparam(ireg,iter,4)   
      rtrunc2=recrparam(ireg,iter,5)   
      acfactr=recrparam(ireg,iter,6)   
      ramplitude=recrparam(ireg,iter,7)   
      rperiod=recrparam(ireg,iter,8)   
      rpahse=recrparam(ireg,iter,9)   
      rinterval=recrparam(ireg,iter,10)   
      rintvcv=recrparam(ireg,iter,11)   
      spasmfct=recrparam(ireg,iter,12)   
      jfunc=irecrcode(ireg,iter,1)
      jdist=irecrcode(ireg,iter,2)
      jtrunc=irecrcode(ireg,iter,3)
      write (*,*) '********************************'
      write (*,*) 'Y/R for recruitment regime: ',ireg
      write (*,*) '********************************'
      if (ioutopt(17).eq.1)  then

      write (17,*) '********************************'
      write (17,*) 'Recruitment regime: ',ireg
      write (17,*) '********************************'
      endif
      if (ioutopt(18).eq.1)  then
      write (18,*) '********************************'
      write (18,*) 'Recruitment regime: ',ireg
      write (18,*) '********************************'
      write (18,*)
      write (18,*) 'F Age 0-maxage'
      endif
      if (ioutopt(19).eq.1)  then
      write (19,*) '********************************'
      write (19,*) 'Recruitment regime: ',ireg
      write (19,*) '********************************'
      write (19,*)
      write (19,*) 'F Age 0-maxage'
      endif
c      rl=rl0
      do 10 ia=0,iaold
c print population data
      if (ioutopt(17).eq.1)write(17,*) 'age sel weca west matur',
     &ia,sel(ia,2),weca(ia),west(ia),rmat(ia)
10    continue  
     
c Prepare a-parameter etc.
      ff1=1.0
      ymax=0.0
      ffymax=0.0
      symax=0.0
      if (ioutopt(17).eq.1) write(17,*)
      if (ioutopt(17).eq.1) write(17,*) 'Yield and SSB per recruit with 
     &deterministic recruitment dependent on SSB: '
      if (spasmfct.gt.1.0) 
     &asrparam=asrorg*((spasmfct-1.0)/rinterval+1.0)
c Find multiplier for the a-parameter taking the skewed truncated distribution 
c into account   
      call truncmult(tm,ireg)
      asrparam=asrparam*tm
c Screen over a range of F-multipliers to get yield/recruit
c and other characteristics of the population
c Stop when SSB/R is close to zero (1% of a-param)
c First, find slope at the origin     
      ff=0.0001
      call eqlib(rn,s,ff,iflgd)
      call yield(rn,y,ff,1)
      slorg=y/0.0001
      ff01=-1.0
c calculate yield/recruit in an initial priming loop.
      if (ioutopt(17).eq.1) write(17,*) 'F-factor  Yield       SSB       
     &TSB   %Virg   Y/SSB Age_catch Age_SSB Recruits'
      ylast=0.0       
      nsteps=int(ff1*100.0)
      do 900 istep=0,nsteps
      ff=float(istep)*0.01
      call eqlib(rn,s,ff,iflgd)
      if (iflgd.eq.1) goto 901
      s1=s
      call calcssba(rn,s,sn,ff,agessb,1)
      call calctsb(rn,tb)
      call yielda(rn,y,cn,ff,agecatch,1)
      if (istep.eq.0) virg=s
      slope=(y-ylast)/0.01
      ylast=y
      if (ioutopt(17).eq.1) write(17,100) ff,y,s,tb,s/virg*100.0,y/s,
     &agecatch,agessb,rn(0)
      if (ioutopt(18).eq.1)write(18,*) ff,(cn(i),i=0,iaold)
      if (ioutopt(19).eq.1)write(19,*) ff,(sn(i),i=0,iaold)
100   format (f6.3,3f10.2,2f8.3,2f8.2,f10.2)      
      if (y.gt.ymax) then
      ymax=y
      ffymax=ff
      symax=s
      endif
      if (ff.gt.0.0.and.slope.lt.slorg*0.1.and.ff01.lt.0.0) then
      ff01=ff
      yy01=y
      endif
900   continue

c Find more precise F01 and Fmax
901   if (ff01.gt.0.0) then
      f01=-1.0
      ylast=yy01
      do 91 istep=0,20
      ff=ff01+float(istep-11)/1000.0
      call eqlib(rn,s,ff,iflgd)
      call calcssba(rn,s,sn,ff,agessb,1)
      call yielda(rn,y,cn,ff,agecatch,1)
      if (istep.gt.0) then
      slope=(y-ylast)/0.001
      if (ff.gt.0.0.and.slope.lt.slorg*0.1.and.f01.lt.0.0) then
      f01=ff
      s01=s
      y01=y
      endif
      endif
      ylast=y
91    continue
      else
      f01=-1.0
      y01=-1.0
      s01=-1.0
      endif

      if (ioutopt(17).eq.1)then
      write(17,111) ffymax
      write(17,115) ymax
      write(17,116) symax
      write(17,*)
      write(17,112) f01
      write(17,113) y01
      write(17,114) s01
      endif
      write(*,*)
      write(*,111) ffymax
      write(*,115) ymax
      write(*,116) symax
      write(*,*)
      write(*,112) f01
      write(*,113) y01
      write(*,114) s01
111   format('F at Ymax (approx): ',f5.2)     
115   format('Yield at Fmax: ',f8.2)
116   format('SSB at Fmax: ',f8.2)
112   format('F0.1: ',f8.3)
113   format('Yield at F0.1: ',f8.2)
114   format('SSB at F0.1: ',f8.2)
      asrparam=asrorg
      rymax(ireg)=ymax
c End screening
      write(*,*)
      write(*,*)
      if (ioutopt(17).eq.1)  then
      write(17,*)
      write(17,*)
      endif
      if (ioutopt(18).eq.1)  then
      write(18,*)
      write(18,*)
      endif
      if (ioutopt(19).eq.1)  then
      write(19,*)
      write(19,*)
      endif
      enddo
c end loop over regimes
      close (17)
      close (18)
      close (19)
      end

      subroutine truncmult(tm,ireg)
c Find mean multiplier caused by truncation      
      include 'hcscom17_1.i'
      real tm,prob(0:100,2)
      integer ireg
      jdist=irecrcode(ireg,iter,2)
      jtrunc=irecrcode(ireg,iter,3)
      sigmar=recrparam(ireg,iter,3)   
      rtrunc1=recrparam(ireg,iter,4)   
      rtrunc2=recrparam(ireg,iter,5)   

c convert rtunc1 to xlow and rtrunc2 to xhigh
c Find tm = mean xsi
      if (jtrunc.eq.1) then
c truncation of std. normal random variable x    
      xlow=rtrunc1
      xhigh=rtrunc2
      else if (jtrunc.eq.2) then
c truncation of final multiplier xm: Translate the bounds to x 
      if (jdist.eq.1) then
c Normal: xm =(1+sigmar*x)      
      xlow=(rtrunc1-1.0)/sigmar
      xhigh=(rtrunc2-1.0)/sigmar
      else if (jdist.eq.2) then
c Lognormal: xm = exp(sigmar*x)      
      xlow=log(rtrunc1)/sigmar
      xhigh=log(rtrunc2)/sigmar
      endif
      endif
      rintv=(xhigh-xlow)/100.0

c scann x over interval, convert each x to xm and cumulate xsi-values and intervals 
      x=xlow+rintv/2.0
      rtp=1.0/sqrt(2.0*3.1415)
c      cumsum=0.0
      do i=1,99
c standard normal
      prob(i,1)=rtp*exp(-(x**2)/2.0)*rintv
c      cumsum=cumsum+prob(i,1)
c convert the x to xm, place the xm-values in prob(i,2)
      if (jtrunc.eq.2) then
      if (jdist.eq.1) then
      prob(i,2)= (1.0+sigmar*x)
      else if (jdist.eq.2) then
      prob(i,2)=exp(sigmar*x)
      endif
      else
      prob(i,2)=1.0+x
      endif
c ready for next step    
      x=x+rintv
      enddo
c prob(.1) now holds the probability of the middle of the interval,
c prob(.2) holds the value of xm in the middle ofthe interval     
      
c calculate the mean:
      
      summ=0.0
      do i=1,99
      summ=summ+prob(i,1)*prob(i,2)
      enddo
      tm=summ
c     tm=summ/cumsum
      end
    
      subroutine eqlib(rn,s,ff,iflgd)
c Find equilibirum SSB and R at F=ff,
c after calculating density dependent weights and maturities
c Also, provides the equilibirum N-values
      include 'hcscom17_1.i'
      real rn(0:maxage),ff,s,sperrec,rc,rnr(0:maxage)
      integer iflgd
      iflgd=0
c calculate stock numbers per recruit in rnr
c These are only dependent on the F
      rnr(0)=1.0
      do ia=0,iaold-2
      rnr(ia+1)=rnr(ia)*exp(-rm(ia)-sel(ia,1)*ff)
      enddo
      rnr(iaold)=
     &rnr(iaold-1)/(1.0-exp(-rm(iaold)-sel(iaold,1)*ff))  
c     write(*,*) ff,rnr(iaold-1),rnr(iaold)
c start with standard weights and maturities
      do ia=0,iaold
      west(ia)=westspec(ia,1)
      rmat(ia)=voksenspec(ia,1)
    
      enddo
      tsb0=-1.0
c get tsb1 according to standard weights (tsb0 negative)
      call nw(tsb0,tsb1,ff,rnr,rc,sperrec,jflg)
      tsb0=tsb1
c get tsb according to the previous tsb
      call nw(tsb0,tsb1,ff,rnr,rc,sperrec,jflg)
c check if tsb has changed.
c If not, break, the standard weights are ok.
c     write(*,*) ff,tsb0,tsb1
      if (abs(log(tsb1/tsb0)).lt.0.0001) goto 90     
c else, check if the new weights remain unchanged
      tsb0=tsb1
      call nw(tsb0,tsb1,ff,rnr,rc,sperrec,jflg)
      if (abs(log(tsb1/tsb0)).lt.0.0001) goto 90     

c if not, search
      n=0
11    call nw(tsb1,tsb2,ff,rnr,rc,sperrec,jflg)
      shift=(tsb2-tsb1)*(tsb1-tsb0)
      if (shift.gt.0.0) then
      tsb0=tsb1
      tsb1=tsb2
      goto 11
      endif
c Shift in sign, the point is somewhere between tsb0 and tsb1
      delta=(tsb1-tsb0)/10.0
      diff1=tsb1-tsb0
      tsb=tsb0-delta
20    tsb=tsb+delta
      n=n+1
      call nw(tsb,tsb1,ff,rnr,rc,sperrec,jflg)
      diff=tsb1-tsb
      if (diff*diff1.le.0.0) delta=-delta/10.0 
      diff1=diff
      if (jflg.eq.1) goto 90
c Temporary: Stop eternal loop
      if (n.gt.100) then
      write(*,*) 'no convergence',tsb0,tsb1,sperrec,rc,west(5),delta
      stop
      endif
c end temporary
      if (abs(delta).gt.0.001) goto 20
c end point when tsb is converged 
90    if (rc.lt.0.0.or.jflg.eq.1) then
      rc=0.0
      iflgd=1
      endif
      s=rc*sperrec
      do ia=0,iaold
      rn(ia)=rnr(ia)*rc
      enddo
c Now er have rn and s
      end    

      subroutine nw(tsbin,tsbout,ff,rnr,rc,sperrec,jflg)
c block of calculations belonging in sub eqlib to 
c derive a new tsbout with weights according to the input tsb
c input: tsbin, leading to weights
c rnr are stock numbers at age per recruit, do not change
c Fishing mortality just to translate rnr to Ns at spawning time
c output: new tsb
c tsb, SSB/recruit and recruitment multiplier based on the weights according to the input tsb
c weights and maturities in west and rmat are commons
      include 'hcscom17_1.i'
      real rn(0:maxage),ff,sperrec,rc,rnr(0:maxage)
      integer jflg
      jflg=0
c Set non-relevant terms to fixed values
      pfactor=1.0
      randfactor=1.0
      yclfct=1.0

c get weights from the input tsb
c Use current weights if tsbin<0
      if (tsbin.ge.0.0) then
      do ia=lowage,iaold
      call calcwema(ia,tsbin,pfactor,randfactor,yclfct,0)
      enddo 
      endif

c Get ssb/recruit with the initial weights and maturities
      sperrec=0.0
      do ia=0,iaold
      sperrec=sperrec+rnr(ia)*exp(-propm*rm(ia)-propf*sel(ia,1)*ff)
     &*west(ia)*rmat(ia)

      enddo
c derive recruitment multiplier from SR function given SSB/recr.
       if (jfunc.eq.1) then
c   hockey stick
       if (sperrec*asrparam.gt.bsrparam) then
       rc=asrparam
       else
       rc=0.0
       endif
      else if (jfunc.eq.2) then
c   Beverton-Holt
      rc=asrparam-bsrparam/sperrec
      if (rc.lt.0.0) rc=0.0
      else if (jfunc.eq.3) then
c   Ricker
      rc=(1.0-log(bsrparam/sperrec/asrparam))*bsrparam/sperrec
      endif
c     write(*,*) ff,asrparam,bsrparam,sperrec,rc
c translate to stock numbers
      if (rc.le.0.0) then

      rc=0.0
      jflg=1
      return
      endif
      do ia=0,iaold
      rn(ia)=rnr(ia)*rc
      enddo
c get tsb
      call calctsb(rn,tsbout)
      end

 
      subroutine prime(rn)
      include 'hcscom17_1.i'
c local variables
      real rn(0:maxage),canum(0:maxage)
      
c if spasmodic recruitments, ensure that the mean R in the priming is right
      if (spasmfct.gt.1.0) 
     &asrparam=asrorg*((spasmfct-1.0)/rinterval+1.0)
      rn(0)=asrparam
      do  ia=1,iaold
      rn(ia)=0.0
      enddo
      if (jfunc.eq.1) then
      s=bsrparam
      else if (jfunc.eq.2) then
      s=bsrparam
      else if (jfunc.eq.3) then
      s=1.0/bsrparam
      endif
      ps=0.0
c run with deterministic recruitment
c for twice iaold, to get the plus group decent as well.
      do 224 i=0,2*iaold
      ff=fprim
      call nextyear(rn,ff,s,1,1,-1)
224   continue

c Then, priming 2*iaold years forwards with stochastic recruitment
c according to recruitment function and weights and maturities according
c to specified variation
      ps=sigmar
      do 223 iiy=1,2*iaold
c Collect canums
      i=iiy-2*iaold
      if (i.ge.iobsstart) then
      do ia=lowage,iaold
      ftemp=ff*sel(ia,1)
      canum(ia)=rn(ia)*ftemp/(ftemp+rm(ia))*(1.0-exp(-ftemp-rm(ia)))
      enddo
      call getobscanum(canum,i)
      endif
      call calcssb(rn,s,fprim,1)
      if (iiy.ge.maxyr-lowage) then
      pastssb(2*iaold-iiy)=s
      endif
      i1=iiy-2*iaold
      call updatewema(i1,rn)
      if (spasmfct.gt.1.0) 
     &asrparam=asrorg*((spasmfct-1.0)/rinterval+1.0)

      call nextyear(rn,fprim,s,1,ispdelay,-1)
c pick up year class factors for weight in the primed year classes
      if (iiy.ge.iaold) wyclfct(i1)=wyclfct0
223   continue
      asrparam=asrorg
c transfer numbers also for starting year of the hcr to rntrue
      end
      


      subroutine bootprint 
      include 'hcscom17_1.i'      
      
c local variables 
      character*20 fmt713
      character*16 fmt711
      character*16 fmt712

      real probcrash(0:maxyr)
      real tall(nityr),pblim(0:maxyr),
     &tacchange(1:maxyr),rectable(0:10,2),x(niter)  
c factors derived from niter 
      rniter = float(niter) 
      ip025 = int(rniter*0.025)
      ip05 = int(rniter*0.05)
      ip975 = int(rniter*0.975)
      ip10 = int(rniter*0.1)
      ip50 = int(rniter*0.5)
      ip90 = int(rniter*0.9)
c formats derived from maxyr, used in printing trajectories
      fmt711='(a10,i5,31f10.0)'      
      write(fmt711(9:10),'(i2)') maxyr+1
      fmt712='(a10,i5,31f10.3)'      
      write(fmt712(9:10),'(i2)') maxyr+1
      fmt713='(a10,i5,10x,  f10.1)'
      write(fmt713(12:13),'(i2)') maxyr
    

      
c fill in iflag: If the flag was set once, that should carry over to 
c the rest of the trajectory, We dont bother about that when calculating means etc,
c but probcrash should reflect the number of trajectories that are lost at some stage.
      do i=1,niter
      do j=2,maxyr      
      iflag(j,i)=max(iflag(j-1,i),iflag(j,i))
      enddo
      enddo
c decide the btrig- the prob of btrig is presented in several places      
      if (irule.eq.2.or.irule.eq.6) then
       btrig=hcrparam(2)
       else
       btrig=hcrparam(1)
       endif

c calculate annual risks to Blim for each year
      do iy=0,maxyr
      do i=1,niter
      tall(i)=truessb(iy,i)
      enddo
c there are two algorithms for the risk, risk and risk2.
c The first is used at present. Both give similar (but not exactly equal) results
      call risk(tall,blim,sigmablim,idistblim,pblim(iy))
      enddo
c Standard table by year  (41 is just an extract with the main table)  
      if (ioutopt(11).eq.1) then
      write(11,*) 'Yearly true mean values'
      write(11,*) 'Year    F      SSB     Catch  TAC  AbsIAV  Ptrig   Pl
     &im   Pcrash   ProcMSY'
      write(41,*) 'Yearly true mean values'
      write(41,*) 'Year    F      SSB     Catch  TAC  AbsIAV  Ptrig   Pl
     &im   Pcrash   ProcMSY'
      c10=0.0
      c50=0.0
      c90=0.0
      f10=0.0
      f50=0.0
      f90=0.0
      
      do 701 iy=0,maxyr
      fmean=0.0
      cmean=0.0
      smean=0.0
      tacmean=0.0
      varmean=0.0
      pbtrig=0.0
      probcrash(iy)=0.0
      rymean=0.0

      
      do 702 i=1,niter
      fmean=fmean+truef(iy,i)/rniter
      cmean=cmean+truecatch(iy,i)/rniter
      smean=smean+truessb(iy,i)/rniter
      tacmean=tacmean+tac(iy,i)/rniter
      if (iy.gt.0) then
      if(tac(iy-1,i).gt.0.0.and.tac(iy,i).gt.0.0) 
     &varmean=varmean+
     &abs((tac(iy,i)-tac(iy-1,i))/(tac(iy-1,i)+tac(iy,i))*2.0/rniter)
      ireg=nregact(iy,i)
      if (rymax(ireg).gt.0.0) 
     &rymean=rymean+tac(iy,i)/rymax(ireg)/rniter
      endif
      if (decbas(iy,1,i).lt.btrig.and.iy.gt.0) pbtrig=pbtrig+1.0/rniter
      probcrash(iy)=probcrash(iy)+float(iflag(iy,i))/rniter
 
702   continue   
c      write(*,*) iy,ireg,rymean



        write(41,101) iy+iy0,fmean,smean,cmean,tacmean,varmean*100.0,
     &pbtrig*100,pblim(iy)*100.0,probcrash(iy)*100.0,rymean*100.0
      write(11,101) iy+iy0,fmean,smean,cmean,tacmean,varmean*100.0,
     &pbtrig*100,pblim(iy)*100.0,probcrash(iy)*100.0,rymean*100.0
701   continue     
101   format (i5,2x,f7.3,3f7.0,5f7.1)    
      write(11,*)
      
c fractiles for main results
      write(11,*) 'Fractiles for TAC'
      write(11,*) 'Year        Mean       10%       50%       90%'
      do 401 iy=0,maxyr
      tacmean=0.0
      do 301 i=1,niter
      tacmean=tacmean+tac(iy,i)/rniter
      tall(i)=tac(iy,i)
301   continue
      call sort(tall,niter)
      write(11,311) iy+iy0,tacmean,tall(ip10),tall(ip50),tall(ip90)      
311   format (i5,2x,4f10.1)
401   continue
      write(11,*)

      write(11,*) 'Fractiles for True F'
      write(11,*) 'Year    Mean    10%    50%    90%'
      do 402 iy=0,maxyr
      fmean=0.0
      do 302 i=1,niter
      fmean=fmean+truef(iy,i)/rniter
      tall(i)=truef(iy,i)
302   continue
      call sort(tall,niter)
      write(11,312) iy+iy0,fmean,tall(ip10),tall(ip50),tall(ip90)      
312   format (i5,2x,4f7.3)
402   continue
      write(11,*)

      write(11,*) 'Fractiles for True SSB'
      write(11,*) 'Year        Mean        5%       50%       90%'
      do 423 iy=0,maxyr
      smean=0.0
      do 323 i=1,niter
      tall(i)=truessb(iy,i)
      smean=smean+truessb(iy,i)/rniter
323   continue
      call sort(tall,niter)
      write(11,311) iy+iy0,smean,tall(ip05),tall(ip50),tall(ip90)  
          
423   continue
      write(11,*)      

      write(11,*) 'Fractiles for True TSB'
      write(11,*) 'Year        Mean       10%       50%       90%'
      do 403 iy=0,maxyr
      tmean=0.0
      do 303 i=1,niter
      tall(i)=truetsb(iy,i)
      tmean=tmean+truetsb(iy,i)/rniter
303   continue
      call sort(tall,niter)
      write(11,311) iy+iy0,tmean,tall(ip10),tall(ip50),tall(ip90)      
403   continue
      write(11,*)      

      write(11,*) 'Fractiles for Recruitment'
      write(11,*) 'Year        Mean       10%       50%       90%'
      do 413 iy=0,maxyr
      rcmean=0.0
      do 313 i=1,niter
      tall(i)=truerecr(iy,i)
      rcmean=rcmean+truerecr(iy,i)/rniter
313   continue
      call sort(tall,niter)
      write(11,311) iy+iy0,rcmean,tall(ip10),tall(ip50),tall(ip90)      
413   continue
      write(11,*)
      
      write(11,*) 'Fractiles for IAV [(TAC(y)-TAC(y-1)]/[(TAC(y-1)+TAC(y
     &))/2] in %'
      write(11,*) 'Year    Mean    10%    50%    90%   Prob50%red'
      do 404 iy=1,maxyr
      varmean=0.0
      red50=0.0
      do 304 i=1,niter
      if (tac(iy-1,i).gt.0.0.and.tac(iy,i).gt.0.0) then
      varmean=varmean+
     &(tac(iy,i)-tac(iy-1,i))/(tac(iy-1,i)+tac(iy,i))*2.0/rniter
      tall(i)=(tac(iy,i)-tac(iy-1,i))/(tac(iy-1,i)+tac(iy,i))*200.0
      if (tac(iy,i).lt.tac(iy-1,i)*0.5) red50=red50+1.0
      else
      tall(i)=0.0
      endif
      if (ioutopt(23).eq.1.and.iy.gt.1) write(23,324) i,iy,tall(i)
324   format(2i5,f7.1)      
      
304   continue
      call sort(tall,niter)
      write(11,333) iy+iy0,varmean*100.0,
     &tall(ip10),tall(ip50),tall(ip90),red50/10.0
404   continue
333   format (i5,2x,5f7.1)
      write(11,*)
      endif
      
      
c Print years intvpr1 - intvpr2
c Note: The fractiles are over the whole material - iterations and years.
c This is different from previous versions, where the entries were the averages
c within each iteration.
      if (ioutopt(12).ge.1) then       
      do iopt=1,ioutopt(12)
      inty1=intvpr1(iopt)
      inty2=intvpr2(iopt)
            
      npryears=inty2-inty1+1
      rnpryears=float(npryears)
      ntall=npryears*niter
      ipy05=int(float(ntall)*0.05)
      ipy10=int(float(ntall)*0.1)
      ipy50=int(float(ntall)*0.5)
      ipy90=int(float(ntall)*0.9)
c true catch
      k=0
      do 810 i=1,niter
      do 810 j=inty1,inty2
      k=k+1
      tall(k)=truecatch(j,i)
810   continue
      call sort(tall,ntall)
      c10=tall(ipy10)
      c50=tall(ipy50)
      c90=tall(ipy90)
c true F   
      k=0   
      do 811 i=1,niter
      do 811 j=inty1,inty2
      k=k+1
      tall(k)=truef(j,i)
811   continue
      call sort(tall,ntall)
      f10=tall(ipy10)
      f50=tall(ipy50)
      f90=tall(ipy90)
c true SSB   
      k=0  
      do 812 i=1,niter
      do 812 j=inty1,inty2
      k=k+1
      tall(k)=truessb(j,i)
812   continue
      call sort(tall,ntall)
      
      s10=tall(ipy10)
      s50=tall(ipy50)
      s90=tall(ipy90)
c true tsb
      k=0
      do 863 i=1,niter
      do 863 j=inty1,inty2
      k=k+1
      tall(k)=truetsb(j,i)
863   continue
      call sort(tall,ntall)
      t10=tall(ipy10)
      t50=tall(ipy50)
      t90=tall(ipy90)
      smean=0.0
      tmean=0.0
      fmean=0.0
      cmean=0.0
      avmean=0.0
      do 813 i=1,niter
      do 814 j=inty1,inty2
c Note: this is maximum risk in all the stated years from year intvpr1 onward
      smean=smean+truessb(j,i)/rnpryears/rniter
      tmean=tmean+truetsb(j,i)/rnpryears/rniter
      fmean=fmean+truef(j,i)/rnpryears/rniter
      cmean=cmean+truecatch(j,i)/rnpryears/rniter      
      if (tac(j-1,i).gt.0.0.and.tac(j,i).gt.0.0) then 
      riav=abs((tac(j,i)-tac(j-1,i))/(tac(j-1,i)+tac(j,i))*2.0)
      avmean=avmean+abs(riav)/rnpryears/rniter
      endif
814   continue
813   continue


c risiki
      risklim=0.0
      risklim2=0.0
      pbtrig=0.0
      do 852 k=inty1,inty2
      irflag1=0
      irflag2=0
      do 851 i=1,niter
      if (decbas(k,1,i).lt.btrig) irflag2=irflag2+1
851   continue 
c Maximum
      if (risklim.lt.pblim(k)) risklim=pblim(k)
      
      if (pbtrig.lt.float(irflag2)/rniter) pbtrig=float(irflag2)/rniter
852   continue

      write(120+iopt,fmtline2) (hcrparam(i),i=1,nruleparam),
     &(hcrparam(i),i=11,26), 
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash(inty2)*100.0
      if (iopt.eq.1) write(*,*) 'Years:',inty1,' - ',inty2,'Cmean:',
     &cmean,'Smean',smean,'Risklim',risklim*100.0

      enddo
      endif
 

c File 13 is  now a file with collected useful information:
c CV of S0, distribution of autocorrelation
      if (ioutopt(13).eq.1) then 

c autocorrelation of true ssb      
      do i=1,niter
      do j=1,maxyr      
      x(j)=truessb(j,i)
      enddo
      call autocorrel(x,1,maxyr,cr)
      tall(i)=cr
      enddo
      call sort(tall,niter)
      acstrue10=tall(ip10)
      acstrue50=tall(ip50)
      acstrue90=tall(ip90)

c acobs is autocorrelation of true ssb minus obs ssb       
      do i=1,niter
      do j=1,maxyr      
      x(j)=truessb(j,i)-obsssb(j,i)
      enddo
      call autocorrel(x,1,maxyr,cr)
      tall(i)=cr
      enddo
      call sort(tall,niter)
      acsobs10=tall(ip10)
      acsobs50=tall(ip50)
      acsobs90=tall(ip90)

c autocorrelation of recruitment      
      do i=1,niter
      do j=1,maxyr      
      x(j)=truerecr(j,i)
      enddo
      call autocorrel(x,1,maxyr,cr)
      tall(i)=cr
      enddo
      call sort(tall,niter)
      acrec10=tall(ip10)
      acrec50=tall(ip50)
      acrec90=tall(ip90)
 
c distribution and cv of ssb in year 0 
 
      do i=1,niter
      tall(i)=truessb(0,i)
      enddo
      call sort(tall,niter)
      s0025=tall(ip025)
      s050=tall(ip50)
      s0975=tall(ip975)
      call avgsd(tall,niter,avgs0,sds0)
      cvs0=0.0
      confint=0.0
      if (avgs0.gt.0.0) cvs0=sds0/avgs0
      if (s050.gt.0.0) confint=(s0975-s0025)/s050
      
      write(13,fmtline3) (hcrparam(i),i=1,nruleparam),
     &(hcrparam(i),i=11,26), 
     &acstrue10,acstrue50,acstrue90,acsobs10,acsobs50,acsobs90,acrec10,
     &acrec50,acrec90,avgs0,s0025,s050,s0975,cvs0,confint

      endif
c end if (ioutopt(13).eq.1)

c All SSB in year 0
      do 600 i=1,niter
      if (ioutopt(21).eq.1) write(21,603) i,truessb(0,i)
c Compare obs and true F-values      
      do 600 iy=0,maxyr
      if (ioutopt(22).eq.1)
     &write(22,602) i,iy,obsf(iy,i),truef(iy,i)
602   format (2i5,2f7.3) 
603   format (i5,f10.0)
600   continue

c print trajectories
      if (ioutopt(15).eq.1) then
      do 710 iter=1,20
      do i=1,maxyr
      if (tac(i-1,iter).gt.0.0) then
      tacchange(i)=((tac(i,iter)-tac(i-1,iter))/tac(i-1,iter)*100.0)
      else
      tacchange(i)=-999.0
      endif
      enddo
      write(15,fmt711) 'TAC       ',iter,(tac(i,iter),i=0,maxyr)
      write(15,fmt711) 'Decis_Bas ',iter,(decbas(i,1,iter),i=0,maxyr)
      write(15,fmt713) 'TACchange ',iter,(tacchange(i),i=1,maxyr)
      write(15,fmt711) 'True_bas  ',iter,(truessb(i,iter),i=0,maxyr)
      write(15,fmt712) 'True_F    ',iter,(truef(i,iter),i=0,maxyr)
      write(15,fmt711) 'Recruits  ',iter,(truerecr(i,iter),i=0,maxyr)
      write(15,fmt711) 'Surpl_prd ',iter,
     &(truetsb(i+1,iter)-truetsb(i,iter)+tac(i,iter),i=0,maxyr-1)
      write(15,*)
710    continue
      endif  
c print recoveries
      if (ioutopt(16).eq.1) then
      do irec=1,10
      rectable(irec,1)=0.0
      rectable(irec,2)=0.0
      enddo
      iy=20
      do iter=1,niter
c Here, recovery after the end of the recruitment failure. which means from year 20 onward      
c is SSB <Blim in year 20?      
      if (truessb(20,iter).lt.blim) then
      rectable(1,1)=rectable(1,1)+1.0
c has it recovered before iy+irec?
      ii=0
      do i=1,10
      if (truessb(iy+i,iter).gt.blim) ii=1
      rectable(i,2)=rectable(i,2)+float(ii)
      enddo
      endif
      enddo
c convert to percentage)
      do i=1,10
      if (rectable(1,1).gt.0.0) then
      rectable(i,2)=rectable(i,2)/rectable(1,1)*100.0
      else
      rectable(i,2)=0.0
      endif
      enddo  
      write(16,'(a33,i10)')  'Number of cases < Blim in year 20',
     &int(rectable(1,1)   )
      write(16,'(a31,10i10)') 'Number (i) of years to recovery',
     &(i,i=1,10)
c      write(16,'(a32,10f10.0)') 'Number of drops below Blim      ',
c     &(rectable(i,1),i=1,5)
      write(16,'(a32,10f10.0)') 'Percent recovery by year i      ',
     &(rectable(i,2),i=1,10)
      write(16,*)
      endif      
      end
 
c**************'      
cc print recoveries
c      if (ioutopt(16).eq.1) then
cc Outer year-loop      
c      write(16,*) 'Year, Drops below Blim=',blim,' in year, % Recovery a
c     &fter years:'
c      write(16,*) '            1     2     3     4    5'
c      do iy=1,maxyr-5
c      
c      do i=0,5
c      rectable(i)=0.0
c      enddo
cc standard blim      
c      do iter=1,niter
cc is there a drop?      
c      if (truessb(iy,iter).lt.blim.and.truessb(iy-1,iter).ge.blim) then
c      rectable(0)=rectable(0)+1.0
cc has it recovered before iy+irec?
c      i=0
c751   i=i+1
c      if (truessb(iy+i,iter).gt.blim) then
c      rectable(i)=rectable(i)+1.0
c      else
c      if (i.lt.5) goto 751
c      endif
c      endif
c      enddo
c      do i=2,5
c      rectable(i)=rectable(i-1)+rectable(i)
c      enddo
c      do i=1,5
c      if (rectable(0).gt.0.0) then
c      rectable(i)=rectable(i)/rectable(0)*100.0
c      else
c      rectable(i)=-1.0
c      endif
c      enddo
c      
c      write(16,'(i6,f6.0,5f6.1)') iy+iy0,(rectable(j),j=0,5)
c      enddo
c      endif
c      end

 
c print weights and maturities: First 10 replicas
      subroutine printwema
      include 'hcscom17_1.i'
      write(25,*) 'Iteration number',iter
      write(25,*)
      write(25,*) 'Catch weights'
      write(25,*) 'Age/Year TSB ',(ia,ia=lowage,iaold)     
      do iy=0,maxyr
      write(25,*) '         ',iy,truetsb(iy,iter),(trueweca(iy,ia),
     &ia=lowage,iaold)
      enddo
      write(25,*)
      write(25,*) 'Stock weights'
      write(25,*) 'Age/Year TSB ',(ia,ia=lowage,iaold)     
      do iy=0,maxyr
      write(25,*) '         ',iy,truetsb(iy,iter),(truewest(iy,ia),
     &ia=lowage,iaold)
      enddo
      write(25,*)
      write(25,*) 'Maturities'
      write(25,*) 'Age/Year  TSB ',(ia,ia=lowage,iaold)     
      do iy=0,maxyr
      write(25,*) '         ',iy,truetsb(iy,iter),(truemat(iy,ia),
     &ia=lowage,iaold)
      enddo
      write(25,*)
   
      end  
      
      subroutine printsr
      include 'hcscom17_1.i'
c local variables
      do i=lowage+1,maxyr
      write(20,100) iter,i,truessb(i-lowage,iter),truen(0,i)
      enddo
100   format(2i5,2f10.0)
      end
      
      subroutine printbas
      include 'hcscom17_1.i'
      character*17 fmt1
      fmt1='(i4,x(2x,2f15.3))'
      write(fmt1(5:5),'(i1)') nbasis
      call compbasis
      write(24,*) 'Iteration',iter
      do iy=0,maxyr
      write(24,fmt1) iy,(truebas(i,iy),decbas(iy,i,iter),i=1,nbasis)
      enddo
      end
      
      subroutine printrecr
      include 'hcscom17_1.i'
      do iy=1,maxyr
      write(26,100) iter,iy,truerecr(iy,iter),obsrecr(iy,iter)
100   format (2i6,2f12.1)      
      enddo
      end
      
      
       
c Library routines
c ********************      
        function snrn(r)
c WHen r is a random number between 0 and 1
c the function returns the number with that probability
c in a standard normal distribution
      real snrn,r
      if (r.gt.0.5) then
      t=sqrt(log((1.0/(r-0.5))**2))
      sg=1.0
      else if (r.lt.0.5) then
      t=sqrt(log((1.0/(r-0.5))**2))
      sg=-1.0
      else
      sg=0.0
      endif
      snrn=sg*(t-(2.30753+0.27061*t)/
     &(1.0+0.99229*t+0.04481*t*t))
      return
      end

       
       function rnx(rintvar)
c returns a random variable between 0 and 1, adjusts the
c seed rintvar
      double precision rintvar,rndn
      real rnx
      rintvar=rintvar*23.0d0
      rintvar=dmod(rintvar,100000001.0d0)
      rndn=rintvar/100000001.0d0
      rnx=sngl(rndn)
      return
      end
      
    
      subroutine multnorm(u,cov,r,rintvar,n)
c generates a random vector of multi-normally distributed
c numbers with given mean vector and covariance matrix
c max dim is 20
      
      integer n
      real covinv(n,n),u(n),r(n),cov(n,n)
      double precision rintvar
      real s1(20),u1(20)

       r01=rnx(rintvar)
      rstn=snrn(r01)
      u1(1)=u(1)
      s1(1)=sqrt(cov(1,1))
      r(1)=u1(1)+rstn*s1(1)
      do 100 i=2,n
      r01=rnx(rintvar)
      rstn=snrn(r01)
      u1(i)=u(i)
      call syminv(cov,covinv,i)

      if (covinv(i,i).le.0.0) then
      write(*,*) 'Covariance martrix not positive definite'
      write(*,*) 'Get a better one!'
      stop
      endif
      
      s1(i)=sqrt(1.0/covinv(i,i))
      do 30 j=1,i-1
      u1(i)=u1(i)-covinv(i,j)/covinv(i,i)*(r(j)-u(j))
30    continue
      r(i)=u1(i)+rstn*s1(i)
100   continue
      end


      subroutine syminv(b,a,n)
c inverts a symmetrical matrix, n*n
c Original source: J.C.Nash,
c Compact numerical methods for computers
c Adam Hilger, Bristol 1979
c Copied from B.F.J.Manly
c Randomisation and Monte Carlo methods in biology
c Chapman & Hall London 1994
c
c The matrix B which is n*n is inverted, The result is placed in a
      real b(20,20),a(20,20),wa(210),wb(20)
c 20 er max. dim, must be adjusted as needed
c wa og wb are working matrices underway, wa must be at least
c  n*(n+1)/2. 
c
c If n=1, just turn the fraction over
      if (n.gt.1) goto 10
      if (b(1,1).eq.0.0) goto 100
      a(1,1)=1.0/b(1,1)
      return
c n>1
10    do 20 i=1,n      
      do 20 j=1,i
20    wa(i*(i-1)/2+j)=b(i,j)
      do 30 ka=1,n
      k=n+1-ka
      s=wa(1)
      m=1
      do 40 i=2,n
      iq=m
      m=m+i
      t=wa(iq+1)
      if (s.eq.0.0) goto 100
      wb(i)=-t/s
      if (i.gt.k) wb(i)=-wb(i)
      do 40 j=iq+2,m
40    wa(j-i)=wa(j)+t*wb(j-iq)
      iq=iq-1
      wa(m)=1.0/s
      do 30 i=2,n
30    wa(iq+i)=wb(i)
c result in a
      do 50 i=1,n
      do 50 j=1,i
      a(i,j)=wa(i*(i-1)/2+j)
50    a(j,i)=a(i,j)
      return
c entry if the matrix is singular
100   stop 'Singular matrix'
      end
 

      subroutine creategeo(geodist,rintv,maxyr)
      real geodist(0:maxyr),rintv
      integer maxyr
      px=0.0
      pr=1.0/rintv
      do i=1,maxyr
      flii=float(i)-1.0
      px=px+pr*exp(flii*log(1.0-pr))
      geodist(i)=px
      enddo
      end

   
      subroutine sort(x,n)
c sorts the first n elements of an array, puts the values in
c the same array      
      include 'hcscom17_1.i'
      real x(nityr),y(nityr)
      y(1)=x(1)
      do 100 i=2,n
      y(i)=0
      do 101 j=1,i
      if (x(i).lt.y(j)) then
      do 102 k=i-1,j,-1
      y(k+1)=y(k)
102   continue      
      y(j)=x(i)
      goto 100
      endif
101   continue
      y(i)=x(i)
100   continue      
      do 104 i=1,n
      x(i)=y(i)
c     write(*,*) i,x(i),y(i)
104   continue     
      end

      subroutine avgsd(x,n,avg,sd)
c calulates mean and sd og the numbers in the array x
c numbers le -999 are not valid
c that has n elements
      integer n
      real x(n)
      real avg,sd
      summ=0.0
      summsq=0.0
      rn=0.0
      do i=1,n
      if (x(i).gt.-999.0) then
      summ=summ+x(i)
      rn=rn+1.0
      endif
      enddo
      avg=summ/rn
      do i=1,n
      if (x(i).gt.-999.0) then
      summsq=summsq+(x(i)-avg)**2
      endif
      enddo
      sd=sqrt(summsq/(rn-1.0))
      end
      
            

      subroutine regress(x,y,a,b,n,ymean)
c Estimation of parameters in a linear regression between
c x and y: y=a+bx
c n is number of obs, limited to niter here
c Etter A. Høyland: 
c Sannsynlighetsregning og statistisk metodelære, bd. 2, s. 179
c Tapir forlag, Trondheim 1973.
      include 'hcscom17_1.i'
      real x(niter),y(niter),a,b,ymean,xmean,xx,xy
      integer n
c mean x and y:
      xmean=0.0
      ymean=0.0
      do 10 i=1,n      
      xmean=xmean+x(i)
      ymean=ymean+y(i)
10    continue
      xmean=xmean/float(n)
      ymean=ymean/float(n)
c     sums xy and xx
      xy=0.0
      xx=0.0
      do 11 i=1,n
      xx=xx+(x(i)-xmean)**2
      xy=xy+(x(i)-xmean)*y(i)
11    continue
      b=xy/xx
      a=ymean-b*xmean
      end

      subroutine autocorrel(x,m,n,acr)
c Estimation of m step autocorrelation acr of a time series x with n terms

      include 'hcscom17_1.i'
      real x(n),y(n),ymean,xmean,xx,yy,xy,acr
      integer n,m
c mean x and y:
      xmean=0.0
      ymean=0.0
      do 10 i=1,n-m
      y(i)=x(i+m)
      xmean=xmean+x(i)
      ymean=ymean+y(i)
10    continue
      xmean=xmean/float(n-m)
      ymean=ymean/float(n-m)
c     sums xy and xx
      xy=0.0
      xx=0.0
      yy=0.0
      do 11 i=1,n-m
      xx=xx+(x(i)-xmean)**2
      yy=yy+(y(i)-ymean)**2
      xy=xy+(x(i)-xmean)*(y(i)-ymean)
11    continue
      acr=0.0
      if(xx*yy.gt.0.0) acr=xy/sqrt(xx*yy)
      end

      subroutine risk(x,a,s,idist,r)
      include 'hcscom17_1.i'
      real x(niter),p,a,s,r
      integer idist
c computes the risk that the variable in x, which is an array of dimension niter) is 
c smaller than the variable represented by a distribution with parameters
c a and s and distribution idist 
c distribution function = 0 or CV = 0: Fixed Blim
c If idist=1: Normal, s is a CV, so sigma=s*a     
c square root of 2*pi
      summ=0.0
c exception
      if (a.le.0.0) then
      write(*,*) 'You ask for the risk to Blim=0' 
      write(*,*) 'That does not make sense - correct the opt.inn file'
      stop
      endif
c Case deterministic Blim      
      if (idist.eq.0.or.s.le.0.0) then
      j=0
      do i=1,niter
      if (x(i).lt.a) j=j+1
      enddo
      r=float(j)/float(niter)     
c Two cases with stochastic Blim      
c calculate the cumulated probability of the value in x(i), with CV = s
c Normal distribution
      else if (idist.eq.1) then   
      sigma=s*a  
      do i=1,niter
      call cumnorm(x(i),a,sigma,p)
       summ=summ+1-p
      enddo
      if (summ.gt.0.0) then
      r=summ/float(niter)
      else
      r=0.0
      endif
c Lognormal      
      else if (idist.eq.2) then
      do i=1,niter
      if (x(i).gt.0.0) then
       y=log(x(i))
       call cumnorm(y,log(a),s,p)
      else
       p=0.0
      endif
      summ=summ+1.0-p
      enddo
      if (summ.gt.0.0) then
      r=summ/float(niter)
      else
      r=0.0
      endif
      endif
c Sum of cumulated probabilities in summ, in both cases.      
      end

      subroutine risk2(x,a,s,idist,r)
c Alternative way of calculating the risk, have a vector of random Blims,
c and compare entry by entry.
c Thanks to Leire Ibaibarriaga.
      include 'hcscom17_1.i'
      real x(niter),y(niter),a,s,r
      integer idist
      if (idist.eq.1) sigma=a*s
      do i=1,niter
c create an array of random Blims
      xr=rnx(rintvar)
      xsi=snrn(xr)
      if (idist.eq.1) then
       y(i)=xsi*sigma+a
      else if (idist.eq.2) then
      y(i)=exp(xsi*s+log(a))
      else if (idist.eq.0) then
      y(i)=a
      endif
      enddo
c Count incidents where x<y
      j=0
      do i=1,niter
      if (x(i).lt.y(i)) j=j+1
c      write(*,*) i,x(i),y(i)
      enddo
      r=float(j)/float(niter)
      end
   

      

      subroutine cumnorm(y,rmean,sigma,prob)
c Returns the cumulated probability of y in a normal distribution with SD = sigma
c Based on an approximation by Bagby 1995, cited in 
c http://mathworld.wolfram.com/NormalDistributionFunction.html, formula 14
c That formula gives the integral from 0 to x,
c rearranged here to give the integral from -inf to x.
  
c  implicit none
      real y,rmean,sigma,x,xx,prob
c convert y to x which is standard normal distributed
      x=(y-rmean)/sigma  
      xx=-x*x
      prob=7.0*exp(0.5*xx)+16.0*exp(0.58558*xx)+(7.0-0.7854*xx)*exp(xx)
      prob=0.5*sqrt(1.0-prob/30.0)
  
      if (x.lt.0.0) then
      prob=0.5-prob
      else
      prob=prob+0.5
      endif
c      write(*,*) y,rmean,x,sigma,prob
      end
       
      subroutine findstring(string,iflag) 
c finds the valid part of an input string
c if the first non-blank character is a number or -, iflag is the 
c position to start. Else, iflag=0: The line can be skipped
      character*200 string
      integer iflag
c get to the first non-blank character   
      i=1
10    if (ichar(string(i:i)).eq.32.and.i.lt.200) then
      i=i+1
      goto 10
      endif
c check the character.  
      iflag=0
      if (i.lt.200) then    
      n=ichar(string(i:i))
      if (n.eq.45.or.n.ge.48.and.n.le.57) iflag=i
      endif
      end
